Type: Package
Package: rjd3bench
Title: Temporal Disaggregation and Benchmarking in 'JDemetra+' 3.x
Version: 3.1.2
Authors@R: c(
      person("Jean", "Palate", email = "palatejean@gmail.com", role = "aut"),
      person("Corentin", "Lemasson", email ="corentin.lemasson@nbb.be", role = c("aut", "cre")),
      person("Tanguy", "Barthelemy", email ="tanguy.barthelemy@insee.fr", role = c("ctb", "art")),
      person("Nikolina", "Rizanovska", email ="nikolina.rizanovska@gov.si", role = "ctb")
  )
Description: Interface to 'JDemetra+' 3.x (<https://github.com/jdemetra>) 
    time series analysis software. It provides a variety of methods for 
    temporal disaggregation & interpolation, benchmarking, reconciliation 
    and calendarization. It incorporates statistical methods described in 
    the latest European Statistical System (ESS) guidelines on temporal 
    disaggregation, benchmarking, and reconciliation (2018 edition). The package 
    implements highly efficient algorithms for fast and reliable computation.
License: EUPL
URL: https://github.com/rjdverse/rjd3bench,
        https://rjdverse.github.io/rjd3bench/
BugReports: https://github.com/rjdverse/rjd3bench/issues
Depends: R (>= 4.1.0)
Imports: rJava (>= 1.0-6), rjd3toolkit (>= 3.7.1), RProtoBuf (>=
        0.4.20), stats, graphics
Encoding: UTF-8
RoxygenNote: 7.3.3
SystemRequirements: Java (>= 21)
Collate: 'utils.R' 'adl.R' 'benchmark.R' 'calendarization.R'
        'deprecated.R' 'mbdenton.R' 'print.R' 'tempdisagg.R' 'zzz.R'
LazyData: TRUE
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-05-04 09:12:03 UTC; lemasso
Author: Jean Palate [aut],
  Corentin Lemasson [aut, cre],
  Tanguy Barthelemy [ctb, art],
  Nikolina Rizanovska [ctb]
Maintainer: Corentin Lemasson <corentin.lemasson@nbb.be>
Repository: CRAN
Date/Publication: 2026-05-04 09:50:02 UTC

# Generated by roxygen2: do not edit by hand

S3method(plot,JD3_ADLDISAGG_RSLTS)
S3method(plot,JD3_INTERP_RSLTS)
S3method(plot,JD3_MBDENTON_RSLTS)
S3method(plot,JD3_TEMPDISAGGI_RSLTS)
S3method(plot,JD3_TEMPDISAGG_RSLTS)
S3method(print,JD3_ADLDISAGG_RSLTS)
S3method(print,JD3_INTERPRAW_RSLTS)
S3method(print,JD3_INTERP_RSLTS)
S3method(print,JD3_MBDENTON_RSLTS)
S3method(print,JD3_TEMPDISAGGI_RSLTS)
S3method(print,JD3_TEMPDISAGGRAW_RSLTS)
S3method(print,JD3_TEMPDISAGG_RSLTS)
S3method(summary,JD3_ADLDISAGG_RSLTS)
S3method(summary,JD3_INTERPRAW_RSLTS)
S3method(summary,JD3_INTERP_RSLTS)
S3method(summary,JD3_MBDENTON_RSLTS)
S3method(summary,JD3_TEMPDISAGGI_RSLTS)
S3method(summary,JD3_TEMPDISAGGRAW_RSLTS)
S3method(summary,JD3_TEMPDISAGG_RSLTS)
export(adl_disaggregation)
export(calendarization)
export(cholette)
export(cubicspline)
export(denton)
export(denton_modelbased)
export(denton_raw)
export(grp)
export(multivariatecholette)
export(temporal_disaggregation)
export(temporal_disaggregation_raw)
export(temporal_interpolation)
export(temporal_interpolation_raw)
export(temporaldisaggregation)
export(temporaldisaggregationI)
import(RProtoBuf)
importFrom(graphics,axis)
importFrom(graphics,legend)
importFrom(graphics,par)
importFrom(rJava,.jarray)
importFrom(rJava,.jcall)
importFrom(rJava,.jcast)
importFrom(rJava,.jcastToArray)
importFrom(rJava,.jclass)
importFrom(rJava,.jevalArray)
importFrom(rJava,.jinstanceof)
importFrom(rJava,.jnew)
importFrom(rJava,.jnull)
importFrom(rJava,.jpackage)
importFrom(rJava,is.jnull)
importFrom(stats,end)
importFrom(stats,frequency)
importFrom(stats,is.ts)
importFrom(stats,start)
importFrom(stats,ts)
importFrom(stats,ts.plot)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# `rjd3bench` <a href="https://rjdverse.github.io/rjd3bench/"><img src="man/figures/logo.png" align="right" height="150" style="float:right; height:150px;"/></a>

<!-- badges: start -->

[![CRAN
status](https://www.r-pkg.org/badges/version/rjd3bench)](https://CRAN.R-project.org/package=rjd3bench)
[![Mentioned in Awesome Official
Statistics](https://awesome.re/mentioned-badge.svg)](https://github.com/SNStatComp/awesome-official-statistics-software)

[![R-CMD-check](https://github.com/rjdverse/rjd3bench/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/rjdverse/rjd3bench/actions/workflows/R-CMD-check.yaml)
[![lint](https://github.com/rjdverse/rjd3bench/actions/workflows/lint.yaml/badge.svg)](https://github.com/rjdverse/rjd3bench/actions/workflows/lint.yaml)

[![GH Pages
built](https://github.com/rjdverse/rjd3bench/actions/workflows/pkgdown.yaml/badge.svg)](https://github.com/rjdverse/rjd3bench/actions/workflows/pkgdown.yaml)
<!-- badges: end -->

## Overview

The package `rjd3bench` provides a variety of methods for temporal
disaggregation, interpolation, benchmarking, reconciliation and
calendarization, using highly efficient algorithms.

Temporal disaggregation & interpolation:

- Chow-Lin, Fernandez and Litterman
- Model-Based Denton
- Autoregressive Distributed Lag (ADL) models
- Reverse regression

Benchmarking:

- Denton
- GRP (Growth Rate Preservation)
- Cubic Splines
- Cholette

Reconciliation and multivariate temporal disaggregation:

- Multivariate Cholette

Calendarization

## Installation

Running rjd3 packages requires **Java 21 or higher**. How to set up such
a configuration in R is explained
[here](https://jdemetra-new-documentation.netlify.app/#Rconfig)

### Latest release

To get the current stable version (from the latest release):

- From CRAN:

``` r
install.packages("rjd3bench")
```

- From GitHub:

``` r
# install.packages("remotes")
remotes::install_github("rjdverse/rjd3toolkit@*release")
remotes::install_github("rjdverse/rjd3bench@*release", build_vignettes = TRUE)
```

- From [r-universe](https://rjdverse.r-universe.dev/rjd3bench):

``` r
install.packages("rjd3bench", repos = c("https://rjdverse.r-universe.dev", "https://cloud.r-project.org"))
```

### Development version

To get the current development version from GitHub:

``` r
# install.packages("remotes")
remotes::install_github("rjdverse/rjd3bench", build_vignettes = TRUE)
```

## Package Maintenance and contributing

Any contribution is welcome and should be done through pull requests
and/or issues. pull requests should include **updated tests** and
**updated documentation**. If functionality is changed, docstrings
should be added or updated.

## Licensing

The code of this project is licensed under the [European Union Public
Licence
(EUPL)](https://interoperable-europe.ec.europa.eu:443/collection/eupl/eupl-text-eupl-12).

Source Code Information
=======================

The java source code of can be found at [https://github.com/jdemetra/jdplus-benchmarking](https://github.com/jdemetra/jdplus-benchmarking).

All other dependencies found in pom.xml of that project can be found via [Maven](https://mvnrepository.com/).

## ----setup, include = FALSE---------------------------------------------------
is_cran <- identical(Sys.getenv("_R_CHECK_CRAN_INCOMING_"), "TRUE")

knitr::opts_chunk$set(
    collapse = TRUE,
    echo = TRUE,
    eval = !is_cran,
    comment = "#>"
)

## -----------------------------------------------------------------------------
library("rjd3bench")
Retail <- rjd3toolkit::Retail
qna_data <- rjd3bench::qna_data

## -----------------------------------------------------------------------------
# Example 1: TD using Chow-Lin to disaggregate annual value added in construction sector using a quarterly indicator
Y <- ts(qna_data$B1G_Y_data[, "B1G_FF"], frequency = 1, start = c(2009, 1))
x <- ts(qna_data$TURN_Q_data[, "TURN_INDEX_FF"], frequency = 4, start = c(2009, 1))
td <- rjd3bench::temporal_disaggregation(Y, indicators = x)

y <- td$estimation$disagg # the disaggregated series
print(td)

# Example 2: interpolation using Fernandez without indicator when the last value (default) of the interpolated series is the one consistent with the low frequency series.
Y <- rjd3toolkit::aggregate(rjd3toolkit::Retail$RetailSalesTotal, 1)
ti <- temporal_interpolation(Y, indicators = NULL, model = "Rw", freq = 4, nfcsts = 2)
y <- ti$estimation$interp # the interpolated series

# Example 3: TD of atypical frequency data using Fernandez with an offset of 1 period
Y <- c(500,510,525,520)
x <- c(97,
       98, 98.5, 99.5, 104, 99,
       100, 100.5, 101, 105.5, 103,
       104.5, 103.5, 104.5, 109, 104,
       107, 103, 108, 113, 110)
td_raw <- temporal_disaggregation_raw(Y, indicators = x, startoffset = 1,  model = "Rw", freqratio = 5)
y <- td_raw$estimation$disagg # the disaggregated series

# Example 4: interpolation of atypical frequency data using Fernandez without offset, when the first value of the interpolated  series is the one consistent with the low frequency series.
Y <- c(500,510,525,520)
x <- c(490, 492.5, 497.5, 520, 495,
       500, 502.5, 505, 527.5, 515,
       522.5, 517.5, 522.5, 545, 520,
       535, 515, 540, 565, 550,
       560)
ti_raw <- temporal_interpolation_raw(Y, indicators = x,  model = "Rw", freqratio = 5, obsposition = 1)
y <- ti_raw$estimation$interp

## -----------------------------------------------------------------------------
# Example: Use of model-based Denton for temporal disaggregation
Y <- ts(qna_data$B1G_Y_data[, "B1G_FF"], frequency = 1, start = c(2009, 1))
x <- ts(qna_data$TURN_Q_data[, "TURN_INDEX_FF"], frequency = 4, start = c(2009, 1))
td_mbd <- denton_modelbased(Y, x, outliers = list("2020-01-01" = 100, "2020-04-01" = 100))

y_mbd <- td_mbd$estimation$disagg
print(td_mbd)

## -----------------------------------------------------------------------------
# Example: Use of ADL models for temporal disaggregation

Y <- ts(qna_data$B1G_Y_data[, "B1G_FF"], frequency = 1, start = c(2009, 1))
x <- ts(qna_data$TURN_Q_data[, "TURN_INDEX_FF"], frequency = 4, start = c(2009, 1))

## 1. without constraints

td_adl <- adl_disaggregation(Y, indicators = x, xar = "FREE")
y <- td_adl$estimation$disagg # the disaggregated series
print(td_adl)

## 2. with constraints

### b1 = -phi * b0 (~ Chow-Lin)
td_adl_constr_1 <- adl_disaggregation(Y, indicators = x, xar = "SAME")

### phi = 1 and b1 = -b0 (~ Fernandez)
td_adl_constr_2 <- adl_disaggregation(Y, constant = FALSE, indicators = x, xar = "SAME", phi = 1, phi.fixed = TRUE)

### b1 = 0 (~ Santos Silva-Cardoso)
td_adl_constr_3 <- adl_disaggregation(Y, indicators = x, xar = "NONE")

## LR test for assessing constraint(s)
LR_stat <- -2 * (td_adl_constr_1$likelihood$ll - td_adl$likelihood$ll) # -> H0 not rejected. Chow-Lin specification is supported here.

## -----------------------------------------------------------------------------
Y <- ts(qna_data$B1G_Y_data[, "B1G_FF"], frequency = 1, start = c(2009, 1))
x <- ts(qna_data$TURN_Q_data[, "TURN_INDEX_FF"], frequency = 4, start = c(2009, 1))
td_rv <- temporaldisaggregationI(Y, indicator = x)

y_rv <- td_rv$estimation$disagg # the disaggregated series
print(td_rv)

## ----eval = FALSE-------------------------------------------------------------
# # Comparison with Chow-Lin
# td_cl <- temporal_disaggregation(Y, indicators = x)
# y_cl <- td_cl$estimation$disagg
# stats::ts.plot(y_rv, y_cl, gpars=list(col=c("red", "blue"), xaxt="n", main="Reverse regression vs Chow-Lin"))
# legend("topleft",c("Reverse regression", "Chow-Lin"), lty = c(1,1), col=c("red", "blue"))

## -----------------------------------------------------------------------------
# Example 1: use of Denton method for benchmarking
Y <- ts(qna_data$B1G_Y_data[, "B1G_HH"], frequency = 1, start = c(2009, 1))

y_den0 <- denton(t = Y, nfreq = 4) # denton PFD without high frequency series

x <- y_den0 + rnorm(n = length(y_den0), mean = 0, sd = 10)
y_den1 <- denton(s = x, t = Y) # denton PFD (= the default)
y_den2 <- denton(s = x, t = Y, d = 2, mul = FALSE) # denton ASD

# Example 2: use of of Denton method for benchmarking atypical frequency data
Y <- c(500,510,525,520)
x <- c(97, 98, 98.5, 99.5, 104,
       99, 100, 100.5, 101, 105.5,
       103, 104.5, 103.5, 104.5, 109,
       104, 107, 103, 108, 113,
       110)

y_denraw <- denton_raw(x, Y, freqratio = 5) # for example, x and Y could be annual and quiquennal series respectively

## -----------------------------------------------------------------------------
# Example: use GRP method for benchmarking
Y <- ts(qna_data$B1G_Y_data[, "B1G_HH"], frequency = 1, start = c(2009, 1))
y_den0 <- denton(t = Y, nfreq = 4)
x <- y_den0 + rnorm(n = length(y_den0), mean = 0, sd = 10)

y_grpf <- grp(s = x, t = Y)
y_grpl <- grp(s = x, t = Y, objective = "Log")

## -----------------------------------------------------------------------------
# Example: use cubic splines for benchmarking
y_cs1 <- cubicspline(t = Y, nfreq = 4) # without high frequency series (smoothing)

x <- y_cs1 + rnorm(n = length(y_cs1), mean = 0, sd = 10)
y_cs2 <- cubicspline(s = x, t = Y) # with a high frequency preliminary series to benchmark

## -----------------------------------------------------------------------------
# Example: use Cholette method for benchmarking
Y <- ts(qna_data$B1G_Y_data[, "B1G_HH"], frequency = 1, start = c(2009, 1))
xn <- c(denton(t = Y, nfreq = 4) + rnorm(n = length(Y)*4, mean = 0, sd = 10), 5750, 5800)
x <- ts(xn, start = start(Y), frequency = 4)

y_cho1 <- cholette(s = x, t = Y, rho = 0.729, lambda = 1, bias = "Multiplicative")  # proportional benchmarking
y_cho2 <- cholette(s = x, t = Y, rho = 0.729, lambda = 1) # proportional benchmarking with no bias (assuming bm=1)
y_cho3 <- cholette(s = x, t = Y, rho = 0.729, lambda = 0, bias = "Additive")  # additive benchmarking
y_cho4 <- cholette(s = x, t = Y, rho = 1, lambda = 1) # Denton PFD
y_cho5 <- cholette(s = x, t = Y, rho = 0, lambda = 0.5) # pro-rating

## -----------------------------------------------------------------------------
# Example: use the multivariate Cholette method for reconciliation
x1 <- ts(c(7, 7.2, 8.1, 7.5, 8.5, 7.8, 8.1, 8.4), frequency = 4, start = c(2010, 1))
x2 <- ts(c(18, 19.5, 19.0, 19.7, 18.5, 19.0, 20.3, 20.0), frequency = 4, start = c(2010, 1))
x3 <- ts(c(1.5, 1.8, 2, 2.5, 2.0, 1.5, 1.7, 2.0), frequency = 4, start = c(2010, 1))

z <- ts(c(27.1, 29.8, 29.9, 31.2, 29.3, 27.9, 30.9, 31.8), frequency = 4, start = c(2010, 1))

Y1 <- ts(c(30.0, 30.6), frequency = 1, start = c(2010, 1))
Y2 <- ts(c(80.0, 81.2), frequency = 1, start = c(2010, 1))
Y3 <- ts(c(8.0, 8.1), frequency = 1, start = c(2010, 1))

### check consistency between temporal and contemporaneous constraints
lfs <- cbind(Y1,Y2,Y3)
as.numeric(rowSums(lfs) - stats::aggregate.ts(z)) # should all be 0

data_list <- list(x1 = x1, x2 = x2, x3 = x3, z = z, Y1 = Y1, Y2 = Y2, Y3 = Y3)
tc <- c("Y1 = sum(x1)", "Y2 = sum(x2)", "Y3 = sum(x3)") # temporal constraints
cc <- c("z = x1+x2+x3") # (binding) contemporaneous constraint
cc_nb <- c("0 = x1+x2+x3-z") # non-binding contemporaneous constraint

rec1 <- multivariatecholette(xlist = data_list, tcvector = tc, ccvector = cc, rho = .5, lambda = .5) # trade-off values for rho and lambda
print(rec1)
rec2 <- multivariatecholette(xlist = data_list, tcvector = tc, ccvector = cc, rho = 1) # Denton
rec3 <- multivariatecholette(xlist = data_list, tcvector = tc, ccvector = cc, rho = 0.729) # Cholette
rec4 <- multivariatecholette(xlist = data_list, tcvector = NULL, ccvector = cc) # no temporal constraints
rec5 <- multivariatecholette(xlist = data_list, tcvector = tc, ccvector = cc_nb) # non-binding contemporaneous constraint

## -----------------------------------------------------------------------------
# Example of calendarization (from Quenneville et al (2012))

## Observed data
obs <- list(
    list(start = "2009-02-18", end = "2009-03-17", value = 9000),
    list(start = "2009-03-18", end = "2009-04-14", value = 5000),
    list(start = "2009-04-15", end = "2009-05-12", value = 9500),
    list(start = "2009-05-13", end = "2009-06-09", value = 7000))

## calendarization in absence of daily indicator values (or weights)
cal_1 <- calendarization(obs, 12, end = "2009-06-30", dailyweights = NULL, stde = TRUE)

ym_1 <- cal_1$rslt
eym_1 <- cal_1$erslt
yd_1 <- cal_1$days
eyd_1 <- cal_1$edays
print(cal_1)

## calendarization in presence of daily indicator values (or weights)
x <- rep(c(1.0, 1.2, 1.8 , 1.6, 0.0, 0.6, 0.8), 19)
cal_2 <- calendarization(obs, 12, end = "2009-06-30", dailyweights = x, stde = TRUE)

ym_2 <- cal_2$rslt
eym_2 <- cal_2$erslt
yd_2 <- cal_2$days
eyd_2 <- cal_2$edays

#' @include utils.R
#' @importFrom stats is.ts
NULL

#' @title Temporal Disaggregation of a Time Series by ADL Model
#'
#' @description
#' Perform temporal disaggregation of low-frequency to high-frequency time
#' series using an Autoregressive Distributed Lag regression model.
#'
#' @param series A low-frequency time series to be disaggregated. It must be `"ts"` object.
#' @param constant Boolean. Indicates whether a constant term is included in the model. The default is `TRUE`.
#' @param trend Boolean. Indicates whether a linear trend is included in the model. The default is `FALSE`.
#' @param indicators One or more high-frequency indicator series. If not NULL (the default), this must be a `"ts"` object or a list of `"ts"` objects.
#' @param average Boolean. Indicates whether an average conversion should be considered. The default is `FALSE`, corresponding to additive conversion.
#' @param phi A numeric value giving the (initial) value of the phi parameter
#' @param phi.fixed Boolean. Specifies whether the supplied value of `phi` is fixed. The default is `FALSE`, which indicates that `phi` is estimated.
#' @param phi.truncated A numeric value defining the lower bound of the admissible range for `phi`.
#' The evaluation range is `[phi.truncated, 1[`.
#' @param xar A character string specifying the constraints imposed on the coefficients of the lagged regression variables. The default is `"FREE"`, which indicates that no constraints are applied. Other options are: `"SAME"`and `"NONE"`.
#' For additional information, see the package vignette.
#' @param diffuse Boolean. Indicates whether the coefficients of the regression model are treated as diffuse (`TRUE`) or as fixed unknown (`FALSE`, the default).
#'
#' @return An object of class "JD3_ADLDISAGG_RSLTS" is returned. The following are returned
#' invisibly as a list:
#' * `regression` `[[1]]` regression coefficients;
#' * `estimation` `[[2]]` disaggregated Time-Series and standard deviation, parameter and residuals;
#' * `likelihood` `[[3]]` likelihood statistics.
#'
#' @references  Proietti, P. (2005). Temporal Disaggregation by State Space Methods: Dynamic Regression Methods Revisited. Working papers and Studies, European Commission, ISSN 1725-4825.
#'
#' @export
#'
#' @seealso For more information, see the vignette:
#'
#' `utils::browseVignettes()`, e.g. `browseVignettes(package = "rjd3bench")`
#'
#' @examplesIf rjd3toolkit::get_java_version() >= rjd3toolkit::minimal_java_version
#' # ADL model
#' data("qna_data")
#' Y <- ts(qna_data$B1G_Y_data[,"B1G_FF"], frequency = 1, start = c(2009,1))
#' x <- ts(qna_data$TURN_Q_data[,"TURN_INDEX_FF"], frequency = 4, start = c(2009,1))
#' td1 <- adl_disaggregation(Y, indicators = x, xar = "FREE")
#' td1$estimation$disagg
#'
#' # ADL models with constraints
#' td2 <- adl_disaggregation(Y, indicators = x, xar = "SAME") # ~ Chow-Lin
#' td3 <- adl_disaggregation(Y, constant = FALSE, indicators = x,
#'                           xar = "SAME", phi = 1, phi.fixed = TRUE) # ~ Fernandez
#' td4 <- adl_disaggregation(Y, indicators = x, xar = "NONE") # ~ Santos Silva-Cardoso
#'
adl_disaggregation <- function(series,
                               constant = TRUE,
                               trend = FALSE,
                               indicators = NULL,
                               average = FALSE,
                               phi = 0.0,
                               phi.fixed = FALSE,
                               phi.truncated = 0.0,
                               xar = c("FREE", "SAME", "NONE"),
                               diffuse = FALSE) {

    xar <- match.arg(xar)
    conversion <- ifelse(average, "Average", "Sum")

    jseries <- rjd3toolkit::.r2jd_tsdata(series)
    jlist <- list()

    if (!is.null(indicators)) {
        if (is.list(indicators)) {
            for (i in seq_along(indicators)) {
                jlist[[i]] <- rjd3toolkit::.r2jd_tsdata(indicators[[i]])
            }
        } else if (stats::is.ts(indicators)) {
            jlist[[1L]] <- rjd3toolkit::.r2jd_tsdata(indicators)
        } else {
            stop("Invalid indicators")
        }
        jindicators <- .jarray(jlist, contents.class = "jdplus/toolkit/base/api/timeseries/TsData")
    } else {
        jindicators <- .jnull("[Ljdplus/toolkit/base/api/timeseries/TsData;")
    }

    jrslt <- .jcall("jdplus/benchmarking/base/r/TemporalDisaggregation", "Ljdplus/benchmarking/base/core/univariate/ADLResults;",
                    "processADL", jseries, constant, trend, jindicators, conversion,
                    phi, phi.fixed, phi.truncated, xar, "TRANSITION", diffuse)

    # Build the S3 result
    bcov <- rjd3toolkit::.proc_matrix(jrslt, "covar")
    vars <- rjd3toolkit::.proc_vector(jrslt, "regnames")
    coef <- rjd3toolkit::.proc_vector(jrslt, "coeff")
    se <- sqrt(diag(bcov))
    t <- coef / se
    m <- data.frame(coef, se, t)
    row.names(m) <- vars

    regression <- list(
        type = xar,
        conversion = conversion,
        model = m,
        cov = bcov
    )
    estimation <- list(
        disagg = rjd3toolkit::.proc_ts(jrslt, "disagg"),
        edisagg = rjd3toolkit::.proc_ts(jrslt, "edisagg"),
        parameter = rjd3toolkit::.proc_numeric(jrslt, "parameter"),
        eparameter = rjd3toolkit::.proc_numeric(jrslt, "eparameter"),
        residuals = .proc_residualsADL(jrslt, diffuse)
    )
    likelihood <- .proc_likelihoodADL(jrslt, diffuse)

    output <- list(
        regression = regression,
        estimation = estimation,
        likelihood = likelihood
    )
    class(output) <- "JD3_ADLDISAGG_RSLTS"
    return(output)
}

#' @include utils.R
NULL

.onAttach <- function(libname, pkgname) {
    if (rjd3toolkit::get_java_version() < rjd3toolkit::minimal_java_version) {
        packageStartupMessage(sprintf("Your java version is %s. %s or higher is needed.",
                                      rjd3toolkit::get_java_version(), rjd3toolkit::minimal_java_version))
    }
}

.onLoad <- function(libname, pkgname) {

    jar_dir <- system.file("java", package = "rjd3bench")
    jars <- list.files(jar_dir, pattern = "\\.jar$", full.names = TRUE)
    rJava::.jaddClassPath(jars)

    result <- rJava::.jpackage(pkgname, lib.loc = libname)
    if (!result) stop("Loading java packages failed", call. = FALSE)

    if (rjd3toolkit::get_java_version() >= rjd3toolkit::minimal_java_version) {
        rjd3toolkit::reload_dictionaries()
    }

    #  proto.dir <- system.file("proto", package = pkgname)
    #  RProtoBuf::readProtoFiles2(protoPath = proto.dir)
}

#' @include utils.R
NULL

#' @export
print.JD3_TEMPDISAGG_RSLTS <- function(x, ...) {
    if (is.null(x$regression$model)) {
        cat("Invalid estimation")
    } else {
        cat("Model:", x$regression$type, "\n")
        print(x$regression$model)

        cat("\n")
        cat("Use summary() for more details. \nUse plot() to see the decomposition of the disaggregated series.")
    }
}

#' @export
print.JD3_TEMPDISAGGRAW_RSLTS <- function(x, ...) {
    if (is.null(x$regression$model)) {
        cat("Invalid estimation")
    } else {
        cat("Model:", x$regression$type, "\n")
        print(x$regression$model)
    }
}

#' @export
print.JD3_INTERP_RSLTS <- function(x, ...) {
    if (is.null(x$regression$model)) {
        cat("Invalid estimation")
    } else {
        cat("Model:", x$regression$type, "\n")
        print(x$regression$model)

        cat("\n")
        cat("Use summary() for more details. \nUse plot() to see the decomposition of the interpolated series.")
    }
}

#' @export
print.JD3_INTERPRAW_RSLTS <- function(x, ...) {
    if (is.null(x$regression$model)) {
        cat("Invalid estimation")
    } else {
        cat("Model:", x$regression$type, "\n")
        print(x$regression$model)
    }
}


#' @export
print.JD3_TEMPDISAGGI_RSLTS <- function(x, ...) {
    if (is.null(x$estimation$parameter)) {
        cat("Invalid estimation")
    } else {
        model <- data.frame(coef = c(round(x$regression$a, 4L), round(x$regression$b, 4L)))
        row.names(model) <- c("a", "b")
        print(model)

        cat("\n")
        cat("Use summary() for more details. \nUse plot() to visualize the disaggregated series.")
    }
}

#' @export
print.JD3_MBDENTON_RSLTS <- function(x, ...) {
    if (is.null(x$estimation$disagg)) {
        cat("Invalid estimation")
    } else {
        cat("Available output:\n")
        print.default(names(x$estimation), ...)

        cat("\n")
        cat("Use summary() for more details.\n",
            "Use plot() to see the disaggregated series and BI ratio",
            "with their respective confidence interval.")
    }
}

#' @export
print.JD3_ADLDISAGG_RSLTS <- function(x, ...) {
    if (is.null(x$regression$model)) {
        cat("Invalid estimation")
    } else {
        cat("Model:", x$regression$type, "\n")
        print(x$regression$model)

        cat("\n")
        cat("Use summary() for more details. \nUse plot() to see the decomposition of the disaggregated series.")
    }
    return(invisible(x))
}

#' @export
summary.JD3_TEMPDISAGG_RSLTS <- function(object, ...) {
    summary_disagg(object)
}

#' @export
summary.JD3_TEMPDISAGGRAW_RSLTS <- function(object, ...) {
    summary_disagg(object)
}

#' @export
summary.JD3_INTERP_RSLTS <- function(object, ...) {
    summary_disagg(object)
}

#' @export
summary.JD3_INTERPRAW_RSLTS <- function(object, ...) {
    summary_disagg(object)
}

#' @export
summary.JD3_MBDENTON_RSLTS <- function(object, ...) {
    if (is.null(object)) {
        cat("Invalid estimation")

    } else {
        cat("\n")
        cat("Likelihood statistics", "\n")
        cat("\n")
        cat("Number of observations: ", object$likelihood$nobs, "\n")
        #cat("Number of effective observations: ", object$likelihood$neffective, "\n")
        cat("Number of estimated parameters: ", object$likelihood$nparams, "\n")
        cat("Standard error: ", "\n")
        cat("AIC: ", object$likelihood$aic, "\n")
        cat("BIC: ", object$likelihood$bic, "\n")

        cat("\n")
        cat("\n")
        cat("Available output:\n")
        print.default(names(object$estimation))
    }
}


#' @export
summary.JD3_ADLDISAGG_RSLTS <- function(object, ...) {
    summary_disagg(object)
}

#' @export
summary.JD3_TEMPDISAGGI_RSLTS <- function(object, ...) {
    if (is.null(object)) {
        cat("Invalid estimation")

    } else {
        cat("\n")
        cat("Likelihood statistics", "\n")
        cat("\n")
        cat("Number of observations: ", object$likelihood$nobs, "\n")
        #cat("Number of effective observations: ", object$likelihood$neffective, "\n")
        cat("Number of estimated parameters: ", object$likelihood$nparams, "\n")
        cat("LogLikelihood: ", object$likelihood$ll, "\n")
        cat("Standard error: ", "\n")
        cat("AIC: ", object$likelihood$aic, "\n")
        cat("BIC: ", object$likelihood$bic, "\n")

        cat("\n")
        cat("\n")
        cat("Model:", object$regression$type, "\n")
        model <- data.frame(coef = c(round(object$regression$a, 4L), round(object$regression$b, 4L)))
        row.names(model) <- c("a", "b")
        print(model)
    }
}

#' @export
#' @importFrom stats ts ts.plot start end
#' @importFrom graphics axis legend
plot.JD3_TEMPDISAGG_RSLTS <- function(x, ...) {
    if (is.null(x)) {
        cat("Invalid estimation")

    } else {
        td_series <- x$estimation$disagg
        reg_effect <- x$estimation$regeffect

        if(is.null(reg_effect)){
            reg_effect <- stats::ts(rep(0, length(td_series)), start =  stats::start(td_series), frequency = stats::frequency(td_series))
            smoothing_effect <- td_series
        }else{
            smoothing_effect <- td_series - reg_effect
        }

        stats::ts.plot(
            td_series, reg_effect, smoothing_effect,
            gpars = list(
                main = "Decomposition",
                col = c("orange", "green", "blue"),
                xlab = "",
                xaxt = "n",
                las = 2L,
                ...
            )
        )
        graphics::axis(side = 1L, at = stats::start(td_series)[1L]:stats::end(td_series)[1L])
        graphics::legend("topleft",
                         c("disaggragated series", "regression effect", "smoothing effect"),
                         lty = 1L,
                         col = c("orange", "green", "blue"),
                         bty = "n",
                         cex = 0.8)
    }
}

#' @export
#' @importFrom stats ts ts.plot start end
#' @importFrom graphics axis legend
plot.JD3_INTERP_RSLTS <- function(x, ...) {
    if (is.null(x)) {
        cat("Invalid estimation")

    } else {
        ti_series <- x$estimation$interp
        reg_effect <- x$estimation$regeffect

        if(is.null(reg_effect)){
            reg_effect <- stats::ts(rep(0, length(ti_series)), start =  stats::start(ti_series), frequency = stats::frequency(ti_series))
            smoothing_effect <- ti_series
        }else{
            smoothing_effect <- ti_series - reg_effect
        }

        stats::ts.plot(
            ti_series, reg_effect, smoothing_effect,
            gpars = list(
                main = "Decomposition",
                col = c("orange", "green", "blue"),
                xlab = "",
                xaxt = "n",
                las = 2L,
                ...
            )
        )
        graphics::axis(side = 1L, at = stats::start(ti_series)[1L]:stats::end(ti_series)[1L])
        graphics::legend("topleft",
                         c("interpolated series", "regression effect", "smoothing effect"),
                         lty = 1L,
                         col = c("orange", "green", "blue"),
                         bty = "n",
                         cex = 0.8)
    }
}


#' @export
#' @importFrom stats ts.plot
#' @importFrom graphics par
plot.JD3_MBDENTON_RSLTS <- function(x, ...) {
    if (is.null(x)) {
        cat("Invalid estimation")
    } else {
        oldpar <- graphics::par(no.readonly = TRUE)
        on.exit(graphics::par(oldpar))

        td <- x$estimation$disagg
        td.sd <- x$estimation$edisagg
        td.lb <- td - 1.96 * td.sd
        td.ub <- td + 1.96 * td.sd

        bi <- x$estimation$biratio
        bi.sd <- x$estimation$ebiratio
        bi.lb <- bi - 1.96 * bi.sd
        bi.ub <- bi + 1.96 * bi.sd

        graphics::par(mfrow = c(2L, 1L))

        stats::ts.plot(
            td, td.lb, td.ub,
            gpars = list(
                main = "Disaggragated series and BI ratio",
                xlab = "",
                ylab = "disaggragated series",
                lty = c(1L, 3L, 3L),
                ...
            )
        )
        stats::ts.plot(
            bi, bi.lb, bi.ub,
            gpars = list(
                xlab = "",
                ylab = "BI ratio",
                lty = c(1L, 3L, 3L),
                ...
            )
        )
    }
}

#' @export
#' @importFrom stats ts.plot start end
#' @importFrom graphics axis legend
plot.JD3_ADLDISAGG_RSLTS <- function(x, ...) {
    if (is.null(x)) {
        cat("Invalid estimation")

    } else {
        td_series <- x$estimation$disagg

        stats::ts.plot(td_series, gpars = list(col = "orange", xlab = "", xaxt = "n", las = 2L, ...))
        graphics::axis(side = 1L, at = stats::start(td_series)[1L]:stats::end(td_series)[1L])
        graphics::legend("topleft", "disaggragated series", lty = 1L, col = "orange", bty = "n", cex = 0.8)
    }
    return(invisible(x))
}

#' @export
#' @importFrom stats ts.plot start end
#' @importFrom graphics axis
plot.JD3_TEMPDISAGGI_RSLTS <- function(x, ...) {
    if (is.null(x)) {
        cat("Invalid estimation")

    } else {
        td_series <- x$estimation$disagg
        stats::ts.plot(td_series, gpars = list(xlab = "", ylab = "disaggragated series", xaxt = "n"))
        graphics::axis(side = 1L, at = stats::start(td_series)[1L]:stats::end(td_series)[1L])
    }
}

#' @include utils.R
NULL

#' @title Temporal Disaggregation and Interpolation of a Time Series using the Model-Based Denton Proportional Method
#'
#' @description
#' The Denton proportional first difference (PFD) method can be expressed as a
#' statistical model in a state-space representation. This formulation provides
#' increased flexibility, including the ability to incorporate outliers, which
#' correspond to level shifts in the Benchmark‑to‑Indicator (BI) ratio, that
#' would otherwise induce unintended wave effects under the standard Denton PFD
#' method. In addition, the approach allows the disaggregated series to be
#' constrained (or 'frozen') at specific periods or prior to a given date by
#' fixing the corresponding high‑frequency BI ratios.
#'
#' @param series A low-frequency time series to be disaggregated or interpolated. It must be either a `"ts"` object or a numeric vector.
#' @param indicator A high-frequency indicator series. It must be of the same class as `series`.
#' @param differencing Not yet implemented. This should be left equal to `1` (corresponding to the Denton PFD method).
#' @param conversion A character string specifying the conversion mode, typically `"Sum"` (the default) or `"Average"`. Other options are: `"Last"`, `"First"` and `"UserDefined"`.
#' @param conversion.obsposition An integer specifying the position of the low-frequency observations within the interpolated series (e.g. the 7th month of the year).
#' This argument is used only for interpolation when `conversion = "UserDefined"`.
#' @param outliers A list specifying the outlier periods and their magnitude.
#'   Each element must be provided as `"YYYY-MM-DD" = value`, where the date
#'   identifies the period. The numeric value specifies the intensity of the
#'   outlier and corresponds to the relative value of the innovation variance
#'   (with `1` indicating the normal situation).
#' @param fixedBIratios A list specifying the periods for which the
#'   Benchmark‑to‑Indicator (BI) ratios should be fixed. Each element must be
#'   provided as `"YYYY-MM-DD" = value`, where the date identifies the period
#'   and the numeric value specifies the fixed BI ratio.
#'
#' @return An object of class "JD3_MBDENTON_RSLTS" is returned. The following are returned invisibly as a list:
#' * `estimation` `[[1]]` disaggregated Time-Series, BI ratios and standard deviations;
#' * `likelihood` `[[2]]` likelihood statistics.
#'
#' @export
#'
#' @seealso For more information, see the vignette:
#'
#' `utils::browseVignettes()`, e.g. `browseVignettes(package = "rjd3bench")`
#'
#' @examplesIf rjd3toolkit::get_java_version() >= rjd3toolkit::minimal_java_version
#' # Retail data, monthly indicator
#' Y <- rjd3toolkit::aggregate(rjd3toolkit::Retail$RetailSalesTotal, 1)
#' x <- rjd3toolkit::aggregate(rjd3toolkit::Retail$FoodAndBeverageStores, 4)
#' td <- denton_modelbased(Y, x, outliers = list("2000-01-01" = 100, "2005-07-01" = 100))
#' y <- td$estimation$edisagg
#'
#' # qna data, quarterly indicator
#' data("qna_data")
#' Y <- ts(qna_data$B1G_Y_data[,"B1G_FF"], frequency = 1, start = c(2009,1))
#' x <- ts(qna_data$TURN_Q_data[,"TURN_INDEX_FF"], frequency = 4, start = c(2009,1))
#'
#' td1 <- denton_modelbased(Y, x)
#' td2 <- denton_modelbased(Y, x, outliers = list("2020-04-01" = 100),
#'                          fixedBIratios = list("2021-04-01" = 39.0))
#' bi1 <- td1$estimation$biratio
#' bi2 <- td2$estimation$biratio
#' y1 <- td1$estimation$disagg
#' y2 <- td2$estimation$disagg
#'
#' stats::ts.plot(bi2, bi1, main = "BI ratios",
#'                gpars = list(col = c("red", "black")))
#' graphics::legend("topright", lty = 1, col = c("black", "red"),
#'                  legend = c("td1", "td2"))
#' stats::ts.plot(y2, y1, main = "Disaggregated series",
#'                gpars = list(col = c("red", "black")))
#' graphics::legend("topleft", lty = 1, col = c("black", "red"),
#'                  legend = c("td1", "td2"))
#'
denton_modelbased <- function(
        series,
        indicator,
        differencing = 1L,
        conversion = c("Sum", "Average", "Last", "First", "UserDefined"),
        conversion.obsposition = 1L,
        outliers = NULL,
        fixedBIratios = NULL) {

    conversion <- match.arg(conversion)

    jseries <- rjd3toolkit::.r2jd_tsdata(series)
    jindicator <- rjd3toolkit::.r2jd_tsdata(indicator)
    if (is.null(outliers)) {
        odates <- .jcast(.jnull(), "[Ljava/lang/String;")
        ovars <- .jnull("[D")
    } else {
        odates <- .jarray(names(outliers))
        ovars <- .jarray(as.numeric(outliers))
    }
    if (is.null(fixedBIratios)) {
        fdates <- .jcast(.jnull(), "[Ljava/lang/String;")
        fvars <- .jnull("[D")
    } else {
        fdates <- .jarray(names(fixedBIratios))
        fvars <- .jarray(as.numeric(fixedBIratios))
    }
    jrslt <- .jcall("jdplus/benchmarking/base/r/TemporalDisaggregation", "Ljdplus/benchmarking/base/core/univariate/ModelBasedDentonResults;",
                    "processModelBasedDenton", jseries, jindicator, 1L, conversion, as.integer(conversion.obsposition), odates, ovars,
                    fdates, fvars)
    # Build the S3 result
    estimation <- list(
        disagg = rjd3toolkit::.proc_ts(jrslt, "disagg"),
        edisagg = rjd3toolkit::.proc_ts(jrslt, "edisagg"),
        biratio = rjd3toolkit::.proc_ts(jrslt, "biratio"),
        ebiratio = rjd3toolkit::.proc_ts(jrslt, "ebiratio")
    )
    likelihood <- rjd3toolkit::.proc_likelihood(jrslt, "likelihood.")

    output <- list(
        estimation = estimation,
        likelihood = likelihood
    )
    class(output) <- "JD3_MBDENTON_RSLTS"

    return(output)
}

#' @include utils.R
NULL

#' @title Calendarization
#'
#' @description
#' Time series data do not always coincide with calendar periods (e.g., fiscal
#' years starting in March-April or retail data collected in non-monthly
#' intervals). Calendarization is the process of transforming the values of a
#' flow time series observed over varying time intervals into values that cover
#' given calendar intervals such as month, quarter or year. The process involves
#' two steps. At first, a state-space representation of the Denton proportional
#' first difference (PFD) method is considered to perform a temporal
#' disaggregation of the observed data into daily values. After that, the
#' resulting daily values are aggregated into the desired calendar reference
#' periods.
#'
#' @param calendarobs A named list containing the observed data. The list must
#'   consist of three elements: `start`, `end` and `value`, where the first two
#'   indicate the starting and ending dates of the observation. See the example.
#' @param freq An integer specifying the annual frequency. If set to `0`, only the daily series is computed.
#' @param start The starting day of the calendarization. This date may precede the first observed data (retropolation).
#' @param end The ending day of the calendarization. This date may exceed the last observed data (extrapolation).
#' @param dailyweights A numeric vector of daily indicator values (or weights).
#'   The vector must have the same length as the requested daily series. When
#'   available, these weights typically reflects daily levels of activity, which
#'   may vary due to seasonality, trading day effects, or other calendar effects
#'   such as public holidays.
#' @param stde Boolean. If `TRUE`, the function also returns the standard errors associated with the results.
#' The default is `FALSE`.
#'
#' @return A list containing the disaggregated daily values, the final aggregated series, and their associated standard errors if requested.
#'
#' @references  Quenneville, B., Picard F., Fortier S. (2012). Calendarization
#'   with interpolating splines and state space models. Statistics Canada, Appl.
#'   Statistics (2013) 62, part 3, pp 371-399.
#'
#' @export
#'
#' @seealso For more information, see the vignette:
#'
#' `utils::browseVignettes()`, e.g. `browseVignettes(package = "rjd3bench")`
#'
#' @examplesIf rjd3toolkit::get_java_version() >= rjd3toolkit::minimal_java_version
#'
#' # Example 1 (from Quenneville et al (2012))
#'
#' ## Observed data
#' obs_1 <- list(
#'     list(start = "2009-02-18", end = "2009-03-17", value = 9000),
#'     list(start = "2009-03-18", end = "2009-04-14", value = 5000),
#'     list(start = "2009-04-15", end = "2009-05-12", value = 9500),
#'     list(start = "2009-05-13", end = "2009-06-09", value = 7000))
#'
#' ## a) calendarization in absence of daily indicator values (or weights)
#' cal_1a <- calendarization(obs_1, 12, end = "2009-06-30", dailyweights = NULL, stde = TRUE)
#'
#' ym_1a <- cal_1a$rslt
#' eym_1a <- cal_1a$erslt
#' yd_1a <- cal_1a$days
#' eyd_1a <- cal_1a$edays
#'
#' ## b) calendarization in presence of daily indicator values (or weights)
#' x <- rep(c(1.0, 1.2, 1.8 , 1.6, 0.0, 0.6, 0.8), 19)
#' cal_1b <- calendarization(obs_1, 12, end = "2009-06-30", dailyweights = x, stde = TRUE)
#'
#' ym_1b <- cal_1b$rslt
#' eym_1b <- cal_1b$erslt
#' yd_1b <- cal_1b$days
#' eyd_1b <- cal_1b$edays
#'
#' # Example 2 (incl. negative value)
#'
#' obs_2 <- list(
#'     list(start = "1980-01-01", end = "1989-12-31", value = 100),
#'     list(start = "1990-01-01", end = "1999-12-31", value = -10),
#'     list(start = "2000-01-01", end = "2002-12-31", value = 50))
#'
#' cal_2 <- calendarization(obs_2, 4, end = "2003-12-31")
#'
#' yq_2 <- cal_2$rslt
#'
calendarization <- function(calendarobs, freq, start = NULL, end = NULL, dailyweights = NULL, stde = FALSE) {
    jcal <- rjd3toolkit::r2jd_calendarts(calendarobs)
    if (is.null(dailyweights)) {
        jw <- .jnull("[D")
    } else {
        jw <- .jarray(as.numeric(dailyweights))
    }
    if (is.null(start)) {
        jstart <- .jnull("java/lang/String")
    } else {
        jstart <- as.character(start)
    }
    if (is.null(end)) {
        jend <- .jnull("java/lang/String")
    } else {
        jend <- as.character(end)
    }
    jrslt <- .jcall("jdplus/benchmarking/base/r/Calendarization", "Ljdplus/benchmarking/base/api/calendarization/CalendarizationResults;",
                    "process", jcal, as.integer(freq), jstart, jend, jw, as.logical(stde))

    if (stde) {
        rslt <- rjd3toolkit::.proc_ts(jrslt, "agg")
        erslt <- rjd3toolkit::.proc_ts(jrslt, "eagg")
        start <- as.Date(rjd3toolkit::.proc_str(jrslt, "start"))
        days <- rjd3toolkit::.proc_vector(jrslt, "days")
        edays <- rjd3toolkit::.proc_vector(jrslt, "edays")
        return(list(rslt = rslt, erslt = erslt, start = start, days = days, edays = edays))
    } else {
        rslt <- rjd3toolkit::.proc_ts(jrslt, "agg")
        start <- as.Date(rjd3toolkit::.proc_str(jrslt, "start"))
        days <- rjd3toolkit::.proc_vector(jrslt, "days")
        return(list(rslt = rslt, start = start, days = days))
    }
}

#' @importFrom rJava .jpackage .jcall .jnull .jarray .jevalArray .jcast .jcastToArray .jinstanceof is.jnull .jnew .jclass
#' @import RProtoBuf
NULL

#' @title Quarterly National Accounts data for temporal disaggregation
#'
#' @description
#' This dataset contains two data frames used for temporal disaggregation and
#' benchmarking exercises. The first data frame, `B1G_Y_data`, includes three
#' annual benchmark series corresponding to Belgian annual value added for the
#' period 2009–2020 in three industries: chemical industry (CE), construction
#' (FF), and transport services (HH). The second data frame, `TURN_Q_data`,
#' contains the corresponding quarterly indicator series derived from VAT-based
#' production indicators, covering the period 2009Q1–2021Q4.
#'
#' @format A named list with two elements:
#' \describe{
#'   \item{`B1G_Y_data`}{A data frame with columns:
#'     \describe{
#'       \item{`DATE`}{Annual periods.}
#'       \item{`B1G_CE`}{Value added for chemical industry.}
#'       \item{`B1G_FF`}{Value added for construction.}
#'       \item{`B1G_HH`}{Value added for transport services.}
#'     }
#'   }
#'   \item{`TURN_Q_data`}{A data frame with columns:
#'     \describe{
#'       \item{`DATE`}{Quarterly periods.}
#'       \item{`TURN_INDEX_CE`}{Quarterly indicator for chemical industry.}
#'       \item{`TURN_INDEX_FF`}{Quarterly indicator for construction.}
#'       \item{`TURN_INDEX_HH`}{Quarterly indicator for transport services.}
#'     }
#'   }
#' }
#'
#' @source Belgian Quarterly National Accounts
#'
#' @examples
#' data(qna_data)
#' names(qna_data)
#' head(qna_data$B1G_Y_data)
#' head(qna_data$TURN_Q_data)
"qna_data"

get_result_item <- function(jd3_obj, item) {
    rslt_item <- tryCatch(rjd3toolkit::result(jd3_obj, item),
                          error = function(err) list(value = NaN, pvalue = NaN))
    if (is.null(rslt_item)) {
        rslt_item <- list(value = NA, pvalue = NA)
    }
    return(rslt_item)
}

get_ll_item_adl <- function(jobj, return_sig, method) {
    rslt_item <- tryCatch(.jcall(jobj, return_sig, method),
                          error = function(err) NA)
    if (is.null(rslt_item)) {
        rslt_item <- NA
    }
    return(rslt_item)
}

### alternatively, we could use proto and move the functions to `rjd3toolkit`
.proc_residuals <- function (jrslt, f){

    z <- rjd3toolkit::.jd3_object(jrslt, "TD", TRUE)

    full_residuals <- get_result_item(z, "residuals.fullresiduals")

    extr_normality <- list(get_result_item(z, "residuals.mean"),
                           get_result_item(z, "residuals.skewness"),
                           get_result_item(z, "residuals.kurtosis"),
                           get_result_item(z, "residuals.doornikhansen"))
    extr_independence <- get_result_item(z, "residuals.ljungbox")
    extr_randomness <- list(get_result_item(z, "residuals.nruns"),
                            get_result_item(z, "residuals.lruns"),
                            get_result_item(z, "residuals.nudruns"),
                            get_result_item(z, "residuals.ludruns"))

    nk <- ifelse(get_result_item(z, "likelihood.nobs") > 2*f+1, 2*f, f)
    linearity_test <- tryCatch(rjd3toolkit::ljungbox(full_residuals^2L, k = nk, lag = 1L, mean = TRUE), error = function(err) NaN)

    normality <- matrix(
        data = unlist(extr_normality),
        nrow = 4L,
        ncol = 2L,
        byrow = TRUE,
        dimnames = list(c("mean", "skewness", "kurtosis",
                          "normality(doornikhansen)"),
                        c("value", "p-value"))
    )
    independence <- matrix(unlist(extr_independence), nrow = 1L, ncol = 2L, byrow = TRUE,
                           dimnames = list("ljung_box", c("value", "p-value")))
    randomness <- matrix(
        data = unlist(extr_randomness),
        nrow = 4L,
        ncol = 2L,
        byrow = TRUE,
        dimnames = list(c("Runs around the mean: number",
                          "Runs around the mean: length",
                          "Up and Down runs: number",
                          "Up and Down runs: length"),
                        c("value", "p-value"))
    )
    linearity <- matrix(
        data = unlist(linearity_test),
        nrow = 1L,
        ncol = 2L,
        byrow = TRUE,
        dimnames = list("ljung_box on squared residuals", c("value", "p-value"))
    )

    return(list(full_residuals = full_residuals,
                tests = list(normality = round(normality, 4L),
                             independence = round(independence, 4L),
                             randomness = round(randomness, 4L),
                             linearity = round(linearity, 4L))))
}

.proc_residualsADL <- function (jrslt, diffuse = FALSE){
    z <- rjd3toolkit::.jd3_object(jrslt, "ADL", TRUE)

    return_sig <- if(diffuse) {
        "Ljdplus/benchmarking/base/core/benchmarking/extractors/MarginalLikelihoodStatistics;"
    } else {
        "Ljdplus/benchmarking/base/core/benchmarking/extractors/ProfileLikelihoodStatistics;"
    }
    method <- if(diffuse) "getMarginalLikelihood" else "getProfileLikelihood"

    jll <- tryCatch(.jcall(z$internal, return_sig, method), error = function(e) NULL)

    return(tryCatch(.jcall(jll, "Ljdplus/toolkit/base/api/data/DoubleSeq;", "getResiduals") |>
                        .jcall("[D", "toArray", simplify = TRUE),
                    error = function(e) NULL))
}

.proc_likelihoodADL <- function (jrslt, diffuse = FALSE){
    z <- rjd3toolkit::.jd3_object(jrslt, "ADL", TRUE)

    return_sig <- if(diffuse) {
        "Ljdplus/benchmarking/base/core/benchmarking/extractors/MarginalLikelihoodStatistics;"
    } else {
        "Ljdplus/benchmarking/base/core/benchmarking/extractors/ProfileLikelihoodStatistics;"
    }
    method <- if(diffuse) "getMarginalLikelihood" else "getProfileLikelihood"

    jll <- tryCatch(.jcall(z$internal, return_sig, method),
                    error = function(e) NULL)

    nparams <- get_ll_item_adl(jll, "I", "getEstimatedParametersCount")
    nobs <- get_ll_item_adl(jll, "I", "getObservationsCount")
    ndiffuse <- get_ll_item_adl(jll, "I", "getDiffuseCount")

    return(list(ll = get_ll_item_adl(jll, "D", "getLogLikelihood"),
                llc = get_ll_item_adl(jll, "D", "getAdjustedLogLikelihood"),
                ssq = get_ll_item_adl(jll, "D", "getSsqErr"),
                nparams = nparams,
                nobs = nobs,
                ndiffuse = ndiffuse,
                df = nobs - nparams - ndiffuse,
                aic = get_ll_item_adl(jll, "D", "aic"),
                aicc = get_ll_item_adl(jll, "D", "aicc"),
                bic = get_ll_item_adl(jll, "D", "bic"),
                hannanquinn = get_ll_item_adl(jll, "D", "hannanQuinn")))
}

summary_disagg <- function(object) {
    if (is.null(object)) {
        cat("Invalid estimation")

    } else {
        cat("\n")
        cat("Likelihood statistics", "\n")
        cat("\n")
        cat("Number of observations: ", object$likelihood$nobs, "\n")
        #cat("Number of effective observations: ", object$likelihood$neffective, "\n") # to fix
        cat("Number of estimated parameters: ", object$likelihood$nparams, "\n")
        cat("LogLikelihood: ", object$likelihood$ll, "\n")
        cat("Standard error: ", "\n")
        cat("AIC: ", object$likelihood$aic, "\n")
        cat("BIC: ", object$likelihood$bic, "\n")

        cat("\n")
        cat("\n")
        cat("Model:", object$regression$type, "\n")
        p <- object$estimation$parameter
        if (! is.nan(p)) {
            cat("Rho :", p, " (", object$estimation$eparameter, ")\n")
            cat("\n")
            cat("\n")
        }
        cat("Regression model", "\n")
        print(object$regression$model)

    }
}

#' @include utils.R
#' @importFrom stats is.ts
NULL

#' @title Deprecated Functions
#'
#' @description
#' This function is deprecated. Use the function `temporal_disaggregation()` or `temporal_interpolation()` instead.
#'
#' @param series,constant,trend,indicators,model,freq,conversion,conversion.obsposition,rho,rho.fixed,rho.truncated,zeroinitialization,diffuse.algorithm,diffuse.regressors Parameters.
#'
#' @returns Return the same value as either function that replaces it.
#'
#' @name deprecated-rjd3bench
#'
#' @export
temporaldisaggregation <- function(
        series,
        constant = TRUE,
        trend = FALSE,
        indicators = NULL,
        model = c("Ar1", "Rw", "RwAr1"),
        freq = 4L,
        conversion = c("Sum", "Average", "Last", "First", "UserDefined"),
        conversion.obsposition = 1L,
        rho = 0.0,
        rho.fixed = FALSE,
        rho.truncated = 0.0,
        zeroinitialization = FALSE,
        diffuse.algorithm = c("SqrtDiffuse", "Diffuse", "Augmented"),
        diffuse.regressors = FALSE) {

    .Deprecated(new = NULL,
                msg = "`temporaldisaggregation()` is deprecated. Use `temporal_disaggregation()` or `temporal_interpolation()` instead.")

    model <- match.arg(model)
    conversion <- match.arg(conversion)
    diffuse.algorithm <- match.arg(diffuse.algorithm)
    if (model != "Ar1" && !zeroinitialization) {
        constant <- FALSE
    }
    jseries <- rjd3toolkit::.r2jd_tsdata(series)
    jlist <- list()
    if (!is.null(indicators)) {
        if (is.list(indicators)) {
            for (i in seq_along(indicators)) {
                jlist[[i]] <- rjd3toolkit::.r2jd_tsdata(indicators[[i]])
            }
        } else if (stats::is.ts(indicators)) {
            jlist[[1L]] <- rjd3toolkit::.r2jd_tsdata(indicators)
        } else {
            stop("Invalid indicators")
        }
        jindicators <- .jarray(jlist, contents.class = "jdplus/toolkit/base/api/timeseries/TsData")
        n_ext <- 0L
    } else {
        jindicators <- .jnull("[Ljdplus/toolkit/base/api/timeseries/TsData;")
        n_ext <- 2 * freq
    }
    jrslt <- .jcall(
        obj = "jdplus/benchmarking/base/r/TemporalDisaggregation",
        returnSig = "Ljdplus/benchmarking/base/core/univariate/TemporalDisaggregationResults;",
        method = "process",
        jseries, constant, trend, jindicators, model, as.integer(freq), as.integer(n_ext),
        conversion, as.integer(conversion.obsposition), rho, rho.fixed, rho.truncated,
        zeroinitialization, diffuse.algorithm, diffuse.regressors
    )

    # Build the S3 result
    bcov <- rjd3toolkit::.proc_matrix(jrslt, "covar")
    vars <- rjd3toolkit::.proc_vector(jrslt, "regnames")
    coef <- rjd3toolkit::.proc_vector(jrslt, "coeff")
    se <- sqrt(diag(bcov))
    t <- coef / se
    m <- data.frame(coef, se, t)
    row.names(m) <- vars

    regression <- list(
        type = model,
        conversion = conversion,
        model = m,
        cov = bcov
    )

    disagg <- rjd3toolkit::.proc_ts(jrslt, "disagg")
    estimation <- list(
        disagg = disagg,
        edisagg = rjd3toolkit::.proc_ts(jrslt, "edisagg"),
        regeffect = rjd3toolkit::.proc_ts(jrslt, "regeffect"),
        smoothingpart = rjd3toolkit::.proc_numeric(jrslt, "smoothingpart"),
        parameter = rjd3toolkit::.proc_numeric(jrslt, "parameter"),
        eparameter = rjd3toolkit::.proc_numeric(jrslt, "eparameter"),
        residuals = .proc_residuals(jrslt, stats::frequency(disagg))
    )
    likelihood <- rjd3toolkit::.proc_likelihood(jrslt, "likelihood.")

    output <- list(
        regression = regression,
        estimation = estimation,
        likelihood = likelihood
    )
    class(output) <- "JD3TempDisagg"
    return(output)
}

#' @include utils.R
NULL

#' @title Benchmarking by means of the Denton Method.
#'
#' @description
#' Denton method relies on the principle of movement preservation. There exist a
#' few variants corresponding to different definitions of movement preservation:
#' additive first difference (AFD), proportional first difference (PFD),
#' additive second difference (ASD), proportional second difference (PSD), etc.
#' The default and most widely used is the Denton PFD method.
#'
#' @param s A preliminary series. If not `NULL`, it must be of the same class as `t`.
#' @param t The low-frequency aggregation constraint. It must be either a `"ts"` object or a numeric vector.
#' @param d An integer specifying the differencing order. The default is `1`.
#' @param mul Boolean. Indicates whether benchmarking is multiplicative (`TRUE`) or additive (`FALSE`). The default is multiplicative.
#' @param nfreq An integer giving the annual frequency of the benchmarked series.
#' This argument is used only when no preliminary series is provided.
#' @param modified Boolean. Specifies whether the modified Denton method (`TRUE`) or the unmodified Denton method (`FALSE`) is applied. The default is `TRUE`.
#' @param conversion A character string specifying the conversion mode, typically `"Sum"` (the default) or `"Average"`. Other options are: `"Last"`, `"First"` and `"UserDefined"`.
#' @param obsposition An integer specifying the position of the observations of the low-frequency constraint within the benchmarked series (e.g. the 7th month of the year).
#' This argument is used only when `conversion = "UserDefined"`.
#' @param nbcsts An integer specifying the number of backcast periods.
#' This argument is ignored when a preliminary series is provided.
#' (Not yet implemented.)
#' @param nfcsts An integer specifying the number of forecast periods.
#' This argument is ignored when a preliminary series is provided.
#' (Not yet implemented.)
#'
#' @return A `"ts"` object with the benchmarked series is returned.
#'
#' @export
#'
#' @seealso For more information, see the vignette:
#'
#' `utils::browseVignettes()`, e.g. `browseVignettes(package = "rjd3bench")`
#'
#' @examplesIf rjd3toolkit::get_java_version() >= rjd3toolkit::minimal_java_version
#' Y <- rjd3toolkit::aggregate(rjd3toolkit::Retail$RetailSalesTotal, 1)
#'
#' # Denton PFD without a preliminary series
#' y1 <- denton(t = Y, nfreq = 4)
#' print(y1)
#'
#' # Denton PFD without a preliminary series and conversion = "Average"
#' denton(t = Y, nfreq = 4, conversion = "Average")
#'
#' # Denton PFD with a preliminary series
#' x <- y1 + rnorm(n = length(y1), mean = 0, sd = 10000)
#' denton(s = x, t = Y)
#'
#' # Denton AFD with a preliminary series
#' denton(s = x, t = Y, mul = FALSE)
#'
denton <- function(s = NULL, t, d = 1L, mul = TRUE, nfreq = 4L, modified = TRUE,
                   conversion = c("Sum", "Average", "Last", "First", "UserDefined"),
                   obsposition = 1L, nbcsts = 0L, nfcsts = 0L) {

    conversion <- match.arg(conversion)

    jd_t <- rjd3toolkit::.r2jd_tsdata(t)

    if (!is.null(s)) {
        jd_s <- rjd3toolkit::.r2jd_tsdata(s)
    } else {
        jd_s <- as.integer(nfreq)
    }
    jd_rslt <- .jcall("jdplus/benchmarking/base/r/Benchmarking", "Ljdplus/toolkit/base/api/timeseries/TsData;", "denton",
                      jd_s, jd_t, as.integer(d), mul, modified, conversion, as.integer(obsposition))
    rjd3toolkit::.jd2r_tsdata(jd_rslt)
}

#' @title Benchmarking of an Atypical Frequency Series by means of the Denton Method.
#'
#' @description
#' Denton method relies on the principle of movement preservation. There exist a
#' few variants corresponding to different definitions of movement preservation:
#' additive first difference (AFD), proportional first difference (PFD),
#' additive second difference (ASD), proportional second difference (PSD), etc.
#' The default and most widely used is the Denton PFD method. The `denton_raw()`
#' function extends `denton()` by allowing benchmarking for any frequency ratio.
#'
#' @param s A preliminary series. If not `NULL`, it must be a numeric vector.
#' @param t The low-frequency aggregation constraint. It must be a numeric vector.
#' @param freqratio An integer specifying the frequency ratio between the benchmarked series and the low-frequency constraint.
#' This argument is mandatory and must be a positive integer.
#' @param d An integer specifying the differencing order. The default is `1`.
#' @param mul Boolean. Indicates whether benchmarking is multiplicative (`TRUE`) or additive (`FALSE`). The default is multiplicative.
#' @param modified Boolean. Specifies whether the modified Denton method (`TRUE`) or the unmodified Denton method (`FALSE`) is applied. The default is `TRUE`.
#' @param conversion  A character string specifying the conversion mode, typically `"Sum"` (the default) or `"Average"`. Other options are: `"Last"`, `"First"` and `"UserDefined"`.
#' @param obsposition An integer specifying the position of the observations of the low-frequency constraint within the benchmarked series (e.g. the 7th month of the year).
#' This argument is used only when `conversion = "UserDefined"`.
#' @param startoffset The number of initial observations in the preliminary series that precede the start of the low-frequency constraint.
#' The value must be either 0 or a positive integer (default is 0).
#' This argument is ignored when no preliminary series is provided.
#' @param nbcsts An integer specifying the number of backcast periods.
#' This argument is ignored when a preliminary series is provided.
#' (Not yet implemented.)
#' @param nfcsts An integer specifying the number of forecast periods.
#' This argument is ignored when a preliminary series is provided.
#' (Not yet implemented.)
#'
#' @return A numeric vector with the benchmarked series is returned.
#'
#' @export
#'
#' @seealso For more information, see the vignette:
#'
#' `utils::browseVignettes()`, e.g. `browseVignettes(package = "rjd3bench")`
#'
#' @examplesIf rjd3toolkit::get_java_version() >= rjd3toolkit::minimal_java_version
#' Y <- c(500,510,525,520)
#' x <- c(97, 98, 98.5, 99.5, 104,
#'        99, 100, 100.5, 101, 105.5,
#'        103, 104.5, 103.5, 104.5, 109,
#'        104, 107, 103, 108, 113,
#'        110)
#'
#' # Denton PFD
#' # for example, x and Y could be annual and quinquennal series respectively
#' denton_raw(x, Y, freqratio = 5)
#'
#' # Denton AFD
#' denton_raw(x, Y, freqratio = 5, mul = FALSE)
#'
#' # Denton PFD without indicator
#' denton_raw(t = Y, freqratio = 2, conversion = "Average")
#'
#' # Denton PFD with/without an offset and conversion = "Last"
#' x2 <- c(485,
#'         490, 492.5, 497.5, 520, 495,
#'         500, 502.5, 505, 527.5, 515,
#'         522.5, 517.5, 522.5, 545, 520,
#'         535, 515, 540, 565, 550)
#' denton_raw(x2, Y, freqratio = 5, conversion = "Last")
#' denton_raw(x2, Y, freqratio = 5, conversion = "Last", startoffset = 1)
#'
#'
denton_raw <- function(s = NULL, t, freqratio, d = 1L, mul = TRUE, modified = TRUE,
					   conversion = c("Sum", "Average", "Last", "First", "UserDefined"),
					   obsposition = 1L, startoffset = 0L, nbcsts = 0L, nfcsts = 0L){

    conversion <- match.arg(conversion)

    if(!(freqratio > 0L && freqratio %% 1L == 0.0)){
        stop("'freqratio' must be a positive integer")
    }
    if(!is.vector(t, mode = "numeric")){
        stop("Aggregation constraint must be a numeric vector")
    }
    if(!(is.vector(s, mode = "numeric") || is.null(s))){
        stop("Preliminary series must be a numeric vector (or NULL)")
    }

    if (!is.null(s)){
        rslt <- .jcall("jdplus/benchmarking/base/r/Benchmarking",  "[D", "dentonRaw",
						as.numeric(s), as.numeric(t), as.integer(freqratio), as.integer(d), mul,
						modified, conversion, as.integer(obsposition), as.integer(startoffset))
    } else {
        rslt <- .jcall("jdplus/benchmarking/base/r/Benchmarking",  "[D", "dentonRaw",
						as.numeric(t), as.integer(freqratio), as.integer(d), mul,
						modified, conversion, as.integer(obsposition), as.integer(startoffset))
    }
    return(rslt)
}

#' @title Benchmarking following the Growth Rate Preservation Principle.
#'
#' @description
#' GRP is a method which explicitly preserves the period-to-period growth rates
#' of the preliminary series. It corresponds to the method of Cauley and Trager
#' (1981), using the solution proposed by Di Fonzo and Marini (2011). BFGS is
#' used as line-search algorithm for the reduced unconstrained minimization
#' problem.
#'
#' @param s A preliminary series. It must be a `"ts"` object.
#' @param t The low-frequency aggregation constraint. It must be a `"ts"` object.
#' @param objective A character string specifying the objective function. The default is `"Forward"`. Other options are: `"Backward"`, `"Symmetric"` and `"Log"`. For additional information on this, see the package vignette.
#' @param conversion A character string specifying the conversion mode, typically `"Sum"` (the default) or `"Average"`. Other options are: `"Last"`, `"First"` and `"UserDefined"`.
#' @param obsposition An integer specifying the position of the observations of the low-frequency constraint within the benchmarked series (e.g. the 7th month of the year).
#' This argument is used only when `conversion = "UserDefined"`.
#' @param eps A numeric value specifying the convergence tolerance. The BFGS
#'   algorithm proceeds until the reduction in the objective function is within
#'   this tolerance (default is `1e-12`) or until the maximum number of iterations is reached.
#' @param iter An integer giving the maximum number of iterations allowed in the BFGS algorithm.
#' The default is `500`.
#' @param dentoninitialization Boolean. Indicates whether the series obtained
#'   via the modified Denton PFD method is used as the starting values for the
#'   GRP optimization procedure. The default is `TRUE`. If `FALSE`, the starting
#'   values are derived directly from the aggregation constraint (e.g. `t/4` for
#'   quarterly series with annual constraint and `conversion = "Sum"`).
#'
#' @return A `"ts"` object with the benchmarked series is returned.
#'
#' @references  Causey, B., and Trager, M.L. (1981). Derivation of Solution to
#'   the Benchmarking Problem: Trend Revision. Unpublished research notes, U.S.
#'   Census Bureau, Washington D.C. Available as an appendix in Bozik and Otto
#'   (1988).
#'
#'   Di Fonzo, T., and Marini, M. (2011). A Newton's Method for Benchmarking
#'   Time Series according to a Growth Rates Preservation Principle. *IMF
#'   WP/11/179*.
#'
#'   Daalmans, J., Di Fonzo, T., Mushkudiani, N. and Bikker, R. (2018). Growth
#'   Rates Preservation (GRP) temporal benchmarking: Drawbacks and alternative
#'   solutions. *Statistics Canada*.
#'
#' @export
#'
#' @seealso For more information, see the vignette:
#'
#' `utils::browseVignettes()`, e.g. `browseVignettes(package = "rjd3bench")`
#'
#' @examplesIf rjd3toolkit::get_java_version() >= rjd3toolkit::minimal_java_version
#' data("qna_data")
#'
#' Y <- ts(qna_data$B1G_Y_data[, "B1G_FF"], frequency = 1, start = c(2009, 1))
#' x <- denton(t = Y, nfreq = 4) + rnorm(n = length(Y) * 4, mean = 0, sd = 10)
#' grp(s = x, t = Y)
#'
grp <- function(s, t,
				objective = c("Forward", "Backward", "Symmetric", "Log"),
                conversion = c("Sum", "Average", "Last", "First", "UserDefined"),
                obsposition = 1L, eps = 1e-12, iter = 500L, dentoninitialization = TRUE) {
  objective <- match.arg(objective)
  conversion <- match.arg(conversion)

  jd_s <- rjd3toolkit::.r2jd_tsdata(s)
  jd_t <- rjd3toolkit::.r2jd_tsdata(t)
  jd_rslt <- .jcall("jdplus/benchmarking/base/r/Benchmarking", "Ljdplus/toolkit/base/api/timeseries/TsData;", "grp",
					jd_s, jd_t, objective, conversion, as.integer(obsposition), eps, as.integer(iter), as.logical(dentoninitialization))
  rjd3toolkit::.jd2r_tsdata(jd_rslt)
}

#' @title Benchmarking by means of Cubic Splines
#'
#' @description
#' Cubic splines are piecewise cubic functions that are linked together in a way
#' to guarantee smoothness at data points. Additivity constraints are added for
#' benchmarking purpose and sub-period estimates are derived from each spline.
#' When a preliminary series is used, cubic splines are no longer drawn based on
#' the low-frequency constraint but the Benchmark-to-Indicator (BI ratio) is the
#' one being smoothed. Sub-period estimates are then simply the product between
#' the smoothed high frequency BI ratio and the preliminary series.
#'
#' @param s A preliminary series. If not `NULL`, it must be of the same class as `t`.
#' @param t The low-frequency aggregation constraint. It must be either an object of class `ts` or a numeric vector.
#' @param nfreq An integer giving the annual frequency of the benchmarked series.
#' This argument is used only when no preliminary series is provided.
#' @param conversion A character string specifying the conversion mode, typically `"Sum"` (the default) or `"Average"`. Other options are: `"Last"`, `"First"` and `"UserDefined"`.
#' @param obsposition An integer specifying the position of the observations of the low-frequency constraint within the benchmarked series (e.g. the 7th month of the year).
#' This argument is used only when `conversion = "UserDefined"`.
#'
#' @return A `"ts"` object with the benchmarked series is returned.
#'
#' @export
#'
#' @seealso For more information, see the vignette:
#'
#' `utils::browseVignettes()`, e.g. `browseVignettes(package = "rjd3bench")`
#'
#' @examplesIf rjd3toolkit::get_java_version() >= rjd3toolkit::minimal_java_version
#' data("qna_data")
#' Y <- ts(qna_data$B1G_Y_data[,"B1G_FF"], frequency = 1, start = c(2009,1))
#'
#' # Cubic spline without preliminary series
#' y1 <- cubicspline(t = Y, nfreq = 4L)
#'
#' # Cubic spline with preliminary series
#' x1 <- y1 + rnorm(n = length(y1), mean = 0, sd = 10)
#' cubicspline(s = x1, t = Y)
#'
#' # Cubic splines used for temporal disaggregation
#' x2 <- ts(qna_data$TURN_Q_data[,"TURN_INDEX_FF"], frequency = 4, start = c(2009,1))
#' cubicspline(s = x2, t = Y)
#'
cubicspline <- function(s = NULL, t, nfreq = 4L,
                        conversion = c("Sum", "Average", "Last", "First", "UserDefined"),
                        obsposition = 1L) {

    conversion <- match.arg(conversion)

    jd_t <- rjd3toolkit::.r2jd_tsdata(t)

    if (!is.null(s)) {
        jd_s <- rjd3toolkit::.r2jd_tsdata(s)
    } else {
        jd_s <- as.integer(nfreq)
    }
    jd_rslt <- .jcall("jdplus/benchmarking/base/r/Benchmarking", "Ljdplus/toolkit/base/api/timeseries/TsData;", "cubicSpline",
                      jd_s, jd_t, conversion, as.integer(obsposition))
    rjd3toolkit::.jd2r_tsdata(jd_rslt)
}


#' @title Benchmarking by means of the Cholette Method
#'
#' @description
#' Cholette method is based on a benchmarking methodology developed at
#' Statistics Canada. It is a generalized method relying on the principle of
#' movement preservation that encompasses other benchmarking methods.
#' The Denton method (both the AFD and PFD variants), as well as the naive
#' pro-rating method, emerge as particular cases of the Cholette method.
#' This method has been widely used for the purpose of benchmarking seasonally
#' adjusted series among others.
#'
#' @param s A preliminary series. It must be of the same class as `t`.
#' @param t The low-frequency aggregation constraint. It must be either a `"ts"` object or a numeric vector.
#' @param rho Numeric. A smoothing parameter whose value must lie between 0 and 1.
#' See the package vignette for more information on the choice of the `rho` parameter.
#' @param lambda Numeric. The adjustment model parameter. Typical choices include `lambda = 1` for proportional benchmarking, `lambda = 0` for additive benchmarking, and `lambda = 0.5` with `rho = 0` for the naive pro-rating method.
#' See the package vignette for more information on the choice of the `lambda` parameter.
#' @param bias Character. Specifies the bias-correction factor. By default, no systematic bias is considered. Other options are: "Additive" and "Multiplicative". See vignette for more details.
#' See the package vignette for more information on the other alternatives.
#' @param conversion A character string specifying the conversion mode, typically `"Sum"` (the default) or `"Average"`. Other options are: `"Last"`, `"First"` and `"UserDefined"`.
#' @param obsposition An integer specifying the position of the observations of the low-frequency constraint within the benchmarked series (e.g. the 7th month of the year).
#' This argument is used only when `conversion = "UserDefined"`.
#'
#' @return A `"ts"` object with the benchmarked series is returned.
#'
#' @references Quenneville, B., Fortier S., Chen Z.-G., Latendresse E. (2006).
#'   Recent Developments in Benchmarking to Annual Totals in X12-ARIMA and at
#'   Statistics Canada. Statistics Canada, Working paper of the Time Series
#'   Research and Analysis Centre.
#'
#' @export
#'
#' @seealso For more information, see the vignette:
#'
#' `utils::browseVignettes()`, e.g. `browseVignettes(package = "rjd3bench")`
#'
#' @examplesIf rjd3toolkit::get_java_version() >= rjd3toolkit::minimal_java_version
#' ym_true <- rjd3toolkit::Retail$RetailSalesTotal
#' yq_true <- rjd3toolkit::aggregate(ym_true, 4)
#' Y_full <- rjd3toolkit::aggregate(ym_true, 1)
#'
#' Y <- window(Y_full, end = c(2009,1)) # say no benchmark yet for the year 2010
#' xm <- ym_true + rnorm(n = length(ym_true), mean = -5000, sd = 10000)
#' xq <- rjd3toolkit::aggregate(xm, 4)
#'
#' # Proportional benchmarking with a bias and some recommended value of rho for
#' # monthly and quarterly series respectively
#' cholette(s = xm, t = Y, rho = 0.9, lambda = 1, bias = "Multiplicative")
#' cholette(s = xq, t = Y, rho = 0.729, lambda = 1, bias = "Multiplicative")
#'
#' # Proportional benchmarking with no bias
#' xm_no_bias <- ym_true + rnorm(n = length(ym_true), mean = 0, sd = 10000)
#' cholette(s = xm_no_bias, t = Y, rho = 0.9, lambda = 1)
#'
#' # Additive benchmarking
#' cholette(s = xm, t = Y, rho = 0.9, lambda = 0, bias = "Additive")
#'
#' # Denton PFD
#' cholette(s = xm, t = Y, rho = 1, lambda = 1)
#'
#' # Pro-rating
#' cholette(s = xm, t = Y, rho = 0, lambda = 0.5)
#'
cholette <- function(s, t, rho = 1., lambda = 1.,
                     bias = c("None", "Additive", "Multiplicative"),
                     conversion = c("Sum", "Average", "Last", "First", "UserDefined"),
                     obsposition = 1L) {
    bias <- match.arg(bias)
    conversion <- match.arg(conversion)

    jd_s <- rjd3toolkit::.r2jd_tsdata(s)
    jd_t <- rjd3toolkit::.r2jd_tsdata(t)
    jd_rslt <- .jcall("jdplus/benchmarking/base/r/Benchmarking", "Ljdplus/toolkit/base/api/timeseries/TsData;", "cholette",
                      jd_s, jd_t, rho, lambda, bias, conversion, as.integer(obsposition))
    rjd3toolkit::.jd2r_tsdata(jd_rslt)
}

#' @title Reconciliation by means of the Multivariate Cholette Method
#'
#' @description
#' This is a multivariate extension of the Cholette benchmarking method which
#' can be used for the purpose of reconciliation. While standard benchmarking
#' methods consider one target series at a time, reconciliation techniques aim
#' to restore consistency in a system of time series with regards to both
#' contemporaneous and temporal constraints. Reconciliation techniques are
#' typically needed when the total and its components are estimated
#' independently (the so-called direct approach). The multivariate Cholette
#' method relies on the principle of movement preservation and encompasses other
#' reconciliation methods such as the multivariate Denton method.
#'
#' @param xlist A named list of `ts` objects containing all input. Each element
#'   should correspond to one input series: a preliminary series, a
#'   low-frequency series representing a temporal aggregation constraint, or a
#'   high-frequency series representing a contemporaneous constraint.
#' @param tcvector A character vector defining the temporal constraints. Each
#'   element must be written in the form `"Y = sum(x)"`, where `"Y"` is the name
#'   of a low-frequency temporal constraint and `"x"` is the name of a
#'   high-frequency preliminary series. The names must match those provided in
#'   `xlist`. The default is `NULL`, indicating that no temporal constraints are
#'   considered.
#' @param ccvector NULL (default) or a character vector defining each contemporaneous constraints. If NULL, no contemporaneous constraint is considered.This is equivalent to applying the univariate Cholette method to each of the preliminary series separately. Otherwise, each element of the vector must be written in the form \eqn{z=w_1 x_1+\ldots+w_n x_n} or \eqn{c=w_1 x_1+\ldots+w_n x_n} where:
#' * \eqn{z} is the name of a high-frequency contemporaneous constraint,
#' * \eqn{(w_1,\ldots,w_n)} are optional numeric weights,
#' * \eqn{(x_1,\ldots,x_n)} are the names of the high-frequency preliminary series and
#' * \eqn{c} is a constant.
#'
#' The \eqn{+} operator can be replaced by \eqn{-}. The names of the contemporaneous constraint(s) and the preliminary series are the one given in the `xlist` argument.
#'
#' \strong{Important}: Any series placed on the left-hand side of a constraint cannot appear on the right-hand side of any other constraint. This is because quantities on the left-hand side are fixed, while those on the right-hand side are adjusted to satisfy the equality.
#' @param rho Numeric. The smoothing parameter whose value must lie between 0
#'   and 1. The default is `0.8`. See the package vignette for more information
#'   on the choice of the `rho` parameter.
#' @param lambda Numeric. The adjustment model parameter. Typical values include
#'   `lambda = 0`, `lambda = 0.5` (the default) and `lambda = 1`. See the package
#'   vignette for more information on the choice of the `lambda` parameter.
#'
#' @return A named list containing the benchmarked series is returned.
#'
#' @export
#'
#' @seealso For more information, see the vignette:
#'
#' `utils::browseVignettes()`, e.g. `browseVignettes(package = "rjd3bench")`
#'
#' @examplesIf rjd3toolkit::get_java_version() >= rjd3toolkit::minimal_java_version
#' # Example 1: one "standard" contemporaneous constraint: x1+x2+x3 = z
#'
#' x1 <- ts(c(7, 7.2, 8.1, 7.5, 8.5, 7.8, 8.1, 8.4), frequency = 4, start = c(2010, 1))
#' x2 <- ts(c(18, 19.5, 19.0, 19.7, 18.5, 19.0, 20.3, 20.0), frequency = 4, start = c(2010, 1))
#' x3 <- ts(c(1.5, 1.8, 2, 2.5, 2.0, 1.5, 1.7, 2.0), frequency = 4, start = c(2010, 1))
#'
#' z <- ts(c(27.1, 29.8, 29.9, 31.2, 29.3, 27.9, 30.9, 31.8), frequency = 4, start = c(2010, 1))
#'
#' Y1 <- ts(c(30.0, 30.6), frequency = 1, start = c(2010, 1))
#' Y2 <- ts(c(80.0, 81.2), frequency = 1, start = c(2010, 1))
#' Y3 <- ts(c(8.0, 8.1), frequency = 1, start = c(2010, 1))
#'
#' ## Check consistency between temporal and contemporaneous constraints
#' lfs <- cbind(Y1,Y2,Y3)
#' rowSums(lfs) - stats::aggregate.ts(z) # should all be 0
#'
#' data_list <- list(x1 = x1, x2 = x2, x3 = x3, z = z, Y1 = Y1, Y2 = Y2, Y3 = Y3)
#' tc <- c("Y1 = sum(x1)", "Y2 = sum(x2)", "Y3 = sum(x3)") # temporal constraints
#' cc <- c("z = x1+x2+x3") # (binding) contemporaneous constraint
#' cc_nb <- c("0 = x1+x2+x3-z") # non-binding contemporaneous constraint
#'
#' ## Run function with default values for rho and lambda
#' multivariatecholette(xlist = data_list, tcvector = tc, ccvector = cc)
#'
#' ## Run function with some trade-off values for rho and lambda
#' multivariatecholette(xlist = data_list, tcvector = tc, ccvector = cc, rho = .5, lambda = .5)
#'
#' ## Run function with the value of rho corresponding to Denton or Cholette
#' multivariatecholette(xlist = data_list, tcvector = tc, ccvector = cc, rho = 1) # Denton
#' multivariatecholette(xlist = data_list, tcvector = tc, ccvector = cc, rho = 0.729) # Cholette
#'
#' ## Run function without temporal constraints
#' multivariatecholette(xlist = data_list, tcvector = NULL, ccvector = cc)
#'
#' ## Run function considering non-binding contemporaneous constraint
#' multivariatecholette(xlist = data_list, tcvector = tc, ccvector = cc_nb)
#'
#' # Example 2: two contemporaneous constraints: x1+3*x2+0.5*x3+x4+x5 = z1 and x1+x2 = x4
#'
#' x1 <- ts(c(7.0,7.3,8.1,7.5,8.5,7.8,8.1,8.4), frequency=4, start=c(2010,1))
#' x2 <- ts(c(1.5,1.8,2.0,2.5,2.0,1.5,1.7,2.0), frequency=4, start=c(2010,1))
#' x3 <- ts(c(18.0,19.5,19.0,19.7,18.5,19.0,20.3,20.0), frequency=4, start=c(2010,1))
#' x4 <- ts(c(8,9.5,9.0,10.7,8.5,10.0,10.3,9.0), frequency=4, start=c(2010,1))
#' x5 <- ts(c(5,9.6,7.2,7.1,4.3,4.6,5.3,5.9), frequency=4, start=c(2010,1))
#'
#' z1 <- ts(c(38.1,41.8,41.9,43.2,38.8,39.1,41.9,43.7), frequency=4, start=c(2010,1))
#'
#' Y1 <- ts(c(30.0,30.5), frequency=1, start=c(2010,1))
#' Y2 <- ts(c(10.0,10.5), frequency=1, start=c(2010,1))
#' Y3 <- ts(c(80.0,81.0), frequency=1, start=c(2010,1))
#' Y4 <- ts(c(40.0,41.0), frequency=1, start=c(2010,1))
#' Y5 <- ts(c(25.0,20.0), frequency=1, start=c(2010,1))
#'
#' ### check consistency between temporal and contemporaneous constraints
#' wlfs <- cbind(Y1,3*Y2,0.5*Y3,Y4,Y5)
#' rowSums(wlfs) - stats::aggregate.ts(z1) # cc1: should all be 0
#' Y1 + Y2 - Y4 # cc2: should all be 0
#'
#' data.list <- list(x1=x1,x2=x2,x3=x3,x4=x4,x5=x5,z1=z1,Y1=Y1,Y2=Y2,Y3=Y3,Y4=Y4,Y5=Y5)
#' tc <- c("Y1=sum(x1)", "Y2=sum(x2)", "Y3=sum(x3)", "Y4=sum(x4)", "Y5=sum(x5)")
#' cc <- c("z1=x1+3*x2+0.5*x3+x4+x5", "0=x1+x2-x4")
#'
#' multivariatecholette(xlist = data.list, tcvector = tc, ccvector = cc)
#'
multivariatecholette <- function(xlist, tcvector = NULL, ccvector = NULL, rho = 0.8, lambda = 0.5) {
    if (!is.list(xlist) || length(xlist) < 3L) {
        stop("incorrect argument, first argument should be a list of at least 3 time series")
    }

    #create the input
    jdic <- .jnew("jdplus/toolkit/base/r/util/Dictionary")
    for (i in seq_along(xlist)){
        .jcall(jdic, "V", "add", names(xlist[i]), rjd3toolkit::.r2jd_tsdata(xlist[[i]]))
    }
    if (is.null(tcvector)) {
        ntc <- 0L
        jtc <- .jcast(.jnull(), "[Ljava/lang/String;")
    } else if (is.vector(tcvector)) {
        ntc <- length(tcvector)
        jtc <- .jarray(tcvector, "java/lang/String")
    } else {
        stop("incorrect argument, constraints should be presented within a character vector")
    }
    if (is.null(ccvector)) {
        ncc <- 0L
        jcc <- .jcast(.jnull(), "[Ljava/lang/String;")
    } else if (is.vector(ccvector)) {
        ncc <- length(ccvector)
        jcc <- .jarray(ccvector, "java/lang/String")
    } else {
        stop("incorrect argument, constraints should be presented within a character vector")
    }
    if (ntc + ncc == 0L) {
        stop("both constraint types are empty, include at least one temporal or contemporaneous constraint")
    }

    jd_rslt <- .jcall("jdplus/benchmarking/base/r/Benchmarking", "Ljdplus/toolkit/base/r/util/Dictionary;", "multiCholette",
                      jdic,  jtc, jcc, rho, lambda)
    if (is.jnull(jd_rslt)) {
        return(NULL)
    }
    rlist <- list()
    rnames <- intersect(names(xlist), .jcall(jd_rslt, "[S", "names"))
    for (i in seq_along(rnames)) {
        jts <- .jcall(jd_rslt, "Ljdplus/toolkit/base/api/timeseries/TsData;", "get", rnames[i])
        if (! is.jnull(jts)) {
            rlist[[rnames[i]]] <- rjd3toolkit::.jd2r_tsdata(jts)
        }
    }
    return(rlist)
}

## ----setup, include = FALSE---------------------------------------------------
is_cran <- identical(Sys.getenv("_R_CHECK_CRAN_INCOMING_"), "TRUE")

knitr::opts_chunk$set(
    collapse = TRUE,
    echo = TRUE,
    eval = !is_cran,
    comment = "#>"
)

## -----------------------------------------------------------------------------
library("rjd3bench")
Retail <- rjd3toolkit::Retail
qna_data <- rjd3bench::qna_data

## -----------------------------------------------------------------------------
# Example 1: TD using Chow-Lin to disaggregate annual value added in construction sector using a quarterly indicator
Y <- ts(qna_data$B1G_Y_data[, "B1G_FF"], frequency = 1, start = c(2009, 1))
x <- ts(qna_data$TURN_Q_data[, "TURN_INDEX_FF"], frequency = 4, start = c(2009, 1))
td <- rjd3bench::temporal_disaggregation(Y, indicators = x)

y <- td$estimation$disagg # the disaggregated series
print(td)

# Example 2: interpolation using Fernandez without indicator when the last value (default) of the interpolated series is the one consistent with the low frequency series.
Y <- rjd3toolkit::aggregate(rjd3toolkit::Retail$RetailSalesTotal, 1)
ti <- temporal_interpolation(Y, indicators = NULL, model = "Rw", freq = 4, nfcsts = 2)
y <- ti$estimation$interp # the interpolated series

# Example 3: TD of atypical frequency data using Fernandez with an offset of 1 period
Y <- c(500,510,525,520)
x <- c(97,
       98, 98.5, 99.5, 104, 99,
       100, 100.5, 101, 105.5, 103,
       104.5, 103.5, 104.5, 109, 104,
       107, 103, 108, 113, 110)
td_raw <- temporal_disaggregation_raw(Y, indicators = x, startoffset = 1,  model = "Rw", freqratio = 5)
y <- td_raw$estimation$disagg # the disaggregated series

# Example 4: interpolation of atypical frequency data using Fernandez without offset, when the first value of the interpolated  series is the one consistent with the low frequency series.
Y <- c(500,510,525,520)
x <- c(490, 492.5, 497.5, 520, 495,
       500, 502.5, 505, 527.5, 515,
       522.5, 517.5, 522.5, 545, 520,
       535, 515, 540, 565, 550,
       560)
ti_raw <- temporal_interpolation_raw(Y, indicators = x,  model = "Rw", freqratio = 5, obsposition = 1)
y <- ti_raw$estimation$interp

## -----------------------------------------------------------------------------
# Example: Use of model-based Denton for temporal disaggregation
Y <- ts(qna_data$B1G_Y_data[, "B1G_FF"], frequency = 1, start = c(2009, 1))
x <- ts(qna_data$TURN_Q_data[, "TURN_INDEX_FF"], frequency = 4, start = c(2009, 1))
td_mbd <- denton_modelbased(Y, x, outliers = list("2020-01-01" = 100, "2020-04-01" = 100))

y_mbd <- td_mbd$estimation$disagg
print(td_mbd)

## -----------------------------------------------------------------------------
# Example: Use of ADL models for temporal disaggregation

Y <- ts(qna_data$B1G_Y_data[, "B1G_FF"], frequency = 1, start = c(2009, 1))
x <- ts(qna_data$TURN_Q_data[, "TURN_INDEX_FF"], frequency = 4, start = c(2009, 1))

## 1. without constraints

td_adl <- adl_disaggregation(Y, indicators = x, xar = "FREE")
y <- td_adl$estimation$disagg # the disaggregated series
print(td_adl)

## 2. with constraints

### b1 = -phi * b0 (~ Chow-Lin)
td_adl_constr_1 <- adl_disaggregation(Y, indicators = x, xar = "SAME")

### phi = 1 and b1 = -b0 (~ Fernandez)
td_adl_constr_2 <- adl_disaggregation(Y, constant = FALSE, indicators = x, xar = "SAME", phi = 1, phi.fixed = TRUE)

### b1 = 0 (~ Santos Silva-Cardoso)
td_adl_constr_3 <- adl_disaggregation(Y, indicators = x, xar = "NONE")

## LR test for assessing constraint(s)
LR_stat <- -2 * (td_adl_constr_1$likelihood$ll - td_adl$likelihood$ll) # -> H0 not rejected. Chow-Lin specification is supported here.

## -----------------------------------------------------------------------------
Y <- ts(qna_data$B1G_Y_data[, "B1G_FF"], frequency = 1, start = c(2009, 1))
x <- ts(qna_data$TURN_Q_data[, "TURN_INDEX_FF"], frequency = 4, start = c(2009, 1))
td_rv <- temporaldisaggregationI(Y, indicator = x)

y_rv <- td_rv$estimation$disagg # the disaggregated series
print(td_rv)

## ----eval = FALSE-------------------------------------------------------------
# # Comparison with Chow-Lin
# td_cl <- temporal_disaggregation(Y, indicators = x)
# y_cl <- td_cl$estimation$disagg
# stats::ts.plot(y_rv, y_cl, gpars=list(col=c("red", "blue"), xaxt="n", main="Reverse regression vs Chow-Lin"))
# legend("topleft",c("Reverse regression", "Chow-Lin"), lty = c(1,1), col=c("red", "blue"))

## -----------------------------------------------------------------------------
# Example 1: use of Denton method for benchmarking
Y <- ts(qna_data$B1G_Y_data[, "B1G_HH"], frequency = 1, start = c(2009, 1))

y_den0 <- denton(t = Y, nfreq = 4) # denton PFD without high frequency series

x <- y_den0 + rnorm(n = length(y_den0), mean = 0, sd = 10)
y_den1 <- denton(s = x, t = Y) # denton PFD (= the default)
y_den2 <- denton(s = x, t = Y, d = 2, mul = FALSE) # denton ASD

# Example 2: use of of Denton method for benchmarking atypical frequency data
Y <- c(500,510,525,520)
x <- c(97, 98, 98.5, 99.5, 104,
       99, 100, 100.5, 101, 105.5,
       103, 104.5, 103.5, 104.5, 109,
       104, 107, 103, 108, 113,
       110)

y_denraw <- denton_raw(x, Y, freqratio = 5) # for example, x and Y could be annual and quiquennal series respectively

## -----------------------------------------------------------------------------
# Example: use GRP method for benchmarking
Y <- ts(qna_data$B1G_Y_data[, "B1G_HH"], frequency = 1, start = c(2009, 1))
y_den0 <- denton(t = Y, nfreq = 4)
x <- y_den0 + rnorm(n = length(y_den0), mean = 0, sd = 10)

y_grpf <- grp(s = x, t = Y)
y_grpl <- grp(s = x, t = Y, objective = "Log")

## -----------------------------------------------------------------------------
# Example: use cubic splines for benchmarking
y_cs1 <- cubicspline(t = Y, nfreq = 4) # without high frequency series (smoothing)

x <- y_cs1 + rnorm(n = length(y_cs1), mean = 0, sd = 10)
y_cs2 <- cubicspline(s = x, t = Y) # with a high frequency preliminary series to benchmark

## -----------------------------------------------------------------------------
# Example: use Cholette method for benchmarking
Y <- ts(qna_data$B1G_Y_data[, "B1G_HH"], frequency = 1, start = c(2009, 1))
xn <- c(denton(t = Y, nfreq = 4) + rnorm(n = length(Y)*4, mean = 0, sd = 10), 5750, 5800)
x <- ts(xn, start = start(Y), frequency = 4)

y_cho1 <- cholette(s = x, t = Y, rho = 0.729, lambda = 1, bias = "Multiplicative")  # proportional benchmarking
y_cho2 <- cholette(s = x, t = Y, rho = 0.729, lambda = 1) # proportional benchmarking with no bias (assuming bm=1)
y_cho3 <- cholette(s = x, t = Y, rho = 0.729, lambda = 0, bias = "Additive")  # additive benchmarking
y_cho4 <- cholette(s = x, t = Y, rho = 1, lambda = 1) # Denton PFD
y_cho5 <- cholette(s = x, t = Y, rho = 0, lambda = 0.5) # pro-rating

## -----------------------------------------------------------------------------
# Example: use the multivariate Cholette method for reconciliation
x1 <- ts(c(7, 7.2, 8.1, 7.5, 8.5, 7.8, 8.1, 8.4), frequency = 4, start = c(2010, 1))
x2 <- ts(c(18, 19.5, 19.0, 19.7, 18.5, 19.0, 20.3, 20.0), frequency = 4, start = c(2010, 1))
x3 <- ts(c(1.5, 1.8, 2, 2.5, 2.0, 1.5, 1.7, 2.0), frequency = 4, start = c(2010, 1))

z <- ts(c(27.1, 29.8, 29.9, 31.2, 29.3, 27.9, 30.9, 31.8), frequency = 4, start = c(2010, 1))

Y1 <- ts(c(30.0, 30.6), frequency = 1, start = c(2010, 1))
Y2 <- ts(c(80.0, 81.2), frequency = 1, start = c(2010, 1))
Y3 <- ts(c(8.0, 8.1), frequency = 1, start = c(2010, 1))

### check consistency between temporal and contemporaneous constraints
lfs <- cbind(Y1,Y2,Y3)
as.numeric(rowSums(lfs) - stats::aggregate.ts(z)) # should all be 0

data_list <- list(x1 = x1, x2 = x2, x3 = x3, z = z, Y1 = Y1, Y2 = Y2, Y3 = Y3)
tc <- c("Y1 = sum(x1)", "Y2 = sum(x2)", "Y3 = sum(x3)") # temporal constraints
cc <- c("z = x1+x2+x3") # (binding) contemporaneous constraint
cc_nb <- c("0 = x1+x2+x3-z") # non-binding contemporaneous constraint

rec1 <- multivariatecholette(xlist = data_list, tcvector = tc, ccvector = cc) # default values for rho and lambda
print(rec1)
rec2 <- multivariatecholette(xlist = data_list, tcvector = tc, ccvector = cc, rho = .5, lambda = .5) # trade-off values for rho and lambda
rec3 <- multivariatecholette(xlist = data_list, tcvector = tc, ccvector = cc, rho = 1) # Denton
rec4 <- multivariatecholette(xlist = data_list, tcvector = tc, ccvector = cc, rho = 0.729) # Cholette
rec5 <- multivariatecholette(xlist = data_list, tcvector = NULL, ccvector = cc) # no temporal constraints
rec6 <- multivariatecholette(xlist = data_list, tcvector = tc, ccvector = cc_nb) # non-binding contemporaneous constraint

## -----------------------------------------------------------------------------
# Example of calendarization (from Quenneville et al (2012))

## Observed data
obs <- list(
    list(start = "2009-02-18", end = "2009-03-17", value = 9000),
    list(start = "2009-03-18", end = "2009-04-14", value = 5000),
    list(start = "2009-04-15", end = "2009-05-12", value = 9500),
    list(start = "2009-05-13", end = "2009-06-09", value = 7000))

## calendarization in absence of daily indicator values (or weights)
cal_1 <- calendarization(obs, 12, end = "2009-06-30", dailyweights = NULL, stde = TRUE)

ym_1 <- cal_1$rslt
eym_1 <- cal_1$erslt
yd_1 <- cal_1$days
eyd_1 <- cal_1$edays
print(cal_1)

## calendarization in presence of daily indicator values (or weights)
x <- rep(c(1.0, 1.2, 1.8 , 1.6, 0.0, 0.6, 0.8), 19)
cal_2 <- calendarization(obs, 12, end = "2009-06-30", dailyweights = x, stde = TRUE)

ym_2 <- cal_2$rslt
eym_2 <- cal_2$erslt
yd_2 <- cal_2$days
eyd_2 <- cal_2$edays

#' @include utils.R
#' @importFrom stats frequency is.ts
NULL

#' @title Temporal Disaggregation of a Time Series by Regression Models.
#'
#' @description
#' Perform temporal disaggregation of low-frequency to high-frequency time
#' series by regression models. The implemented models include Chow-Lin,
#' Fernandez, Litterman and some variants of those algorithms.
#'
#' @param series A low-frequency time series to be disaggregated. It must be a `"ts"` object.
#' @param constant Boolean. Indicates whether a constant term is included in the model. The default is `TRUE`.
#' Note that this argument is used only with `model = "Ar1"` when `zeroinitialization = FALSE`. For additional information, see the package vignette.
#' @param trend Boolean. Indicates whether a linear trend is included in the model. The default is `FALSE`.
#' @param indicators One or more high-frequency indicator series used in the temporal disaggregation.
#' If `NULL` (the default), no indicator is used. When provided, the argument must be a `"ts"` object or a list of `"ts"` objects.
#' @param model A character string specifying the model of the error term at the disaggregated level.
#' The options are: `"Ar1"` (Chow Lin, the default), `"Rw"` (Fernandez), and `"RwAr1"` (Litterman).
#' @param freq An integer giving the annual frequency of the disaggregated series.
#' This argument is ignored when one or more indicator series is provided.
#' @param average Boolean. Indicates whether an average conversion should be considered. The default is `FALSE`, corresponding to additive conversion.
#' @param rho A numeric value giving the (initial) value of the autoregressive parameter.
#' This argument is used only for `"Ar1"` and `"RwAr1"` models.
#' @param rho.fixed Boolean. Specifies whether the supplied value of `rho` is fixed. The default is `FALSE`, which indicates that `rho` is estimated.
#' @param rho.truncated A numeric value defining the lower bound of the admissible range for `rho`.
#' The evaluation range is `[rho.truncated, 1[`.
#' @param zeroinitialization Boolean. If `TRUE`, the initial values of the autoregressive model are set to zero. The default is `FALSE`.
#' @param diffuse.algorithm A character string specifying the algorithm used for diffuse initialization. The default is `"SqrtDiffuse"`. Other options are: `"Diffuse"` and `"Augmented"`.
#' @param diffuse.regressors Boolean. Indicates whether the coefficients of the regression model are treated as diffuse (`TRUE`) or as fixed unknown (`FALSE`, the default).
#' @param nbcsts An integer specifying the number of backcast periods.
#' This argument is ignored when one or more indicator series is provided.
#' @param nfcsts An integer specifying the number of forecast periods.
#' This argument is ignored when one or more indicator series is provided.
#'
#' @return An object of class "JD3_TEMPDISAGG_RSLTS" is returned. The following are returned
#' invisibly as a list:
#' * `regression` `[[1]]` regression coefficients;
#' * `estimation` `[[2]]` disaggregated Time-Series and standard deviation, regression effects, smoothing part, parameter and residuals;
#' * `likelihood` `[[3]]` likelihood statistics.
#'
#' @export
#'
#' @seealso `temporal_interpolation()` for interpolation,
#'
#' `temporal_disaggregation_raw()` for temporal disaggregation of atypical frequency series,
#'
#' `temporal_interpolation_raw()` for interpolation of atypical frequency series
#'
#'
#' For more information, see the vignette:
#'
#' `utils::browseVignettes()`, e.g. `browseVignettes(package = "rjd3bench")`
#'
#' @examplesIf rjd3toolkit::get_java_version() >= rjd3toolkit::minimal_java_version
#' # Chow-lin with a monthly indicator
#' Y <- rjd3toolkit::aggregate(rjd3toolkit::Retail$RetailSalesTotal, 1)
#' x <- rjd3toolkit::Retail$FoodAndBeverageStores
#' td <- temporal_disaggregation(Y, indicators = x)
#' td$estimation$disagg
#'
#' # Fernandez with and without a quarterly indicator
#' data("qna_data")
#' Y <- ts(qna_data$B1G_Y_data[,"B1G_FF"], frequency = 1, start = c(2009,1))
#' x <- ts(qna_data$TURN_Q_data[,"TURN_INDEX_FF"], frequency = 4, start = c(2009,1))
#' td1 <- temporal_disaggregation(Y, indicators = x, model = "Rw")
#' td1$estimation$disagg
#'
#' td2 <- temporal_disaggregation(Y, model = "Rw", nfcsts = 6)
#' td2$estimation$disagg
#'
#' # Chow-lin applied to index series
#' Y_index <- 100 * Y / Y[1]
#' x_index <- 100 * x / x[1]
#' td3 <- temporal_disaggregation(Y, indicators = x, average = TRUE)
#' td3$estimation$disagg
#'
temporal_disaggregation <- function(
        series,
        constant = TRUE,
        trend = FALSE,
        indicators = NULL,
        model = c("Ar1", "Rw", "RwAr1"),
        freq = 4L,
        average = FALSE,
        rho = 0.0,
        rho.fixed = FALSE,
        rho.truncated = 0.0,
        zeroinitialization = FALSE,
        diffuse.algorithm = c("SqrtDiffuse", "Diffuse", "Augmented"),
        diffuse.regressors = FALSE,
        nbcsts = 0L,
        nfcsts = 0L) {

	model <- match.arg(model)
    diffuse.algorithm <- match.arg(diffuse.algorithm)
    if (model != "Ar1" && !zeroinitialization) {
        constant <- FALSE
    }

    jseries <- rjd3toolkit::.r2jd_tsdata(series)
	if (!is.null(indicators)) {
	    jlist <- list()
        if (is.list(indicators)) {
            for (i in seq_along(indicators)) {
                jlist[[i]] <- rjd3toolkit::.r2jd_tsdata(indicators[[i]])
            }
        } else if (stats::is.ts(indicators)) {
            jlist[[1L]] <- rjd3toolkit::.r2jd_tsdata(indicators)
        } else {
            stop("Invalid indicators")
        }
        jindicators <- .jarray(jlist, contents.class = "jdplus/toolkit/base/api/timeseries/TsData")

        jrslt <- .jcall(
            obj = "jdplus/benchmarking/base/r/TemporalDisaggregation",
            returnSig = "Ljdplus/benchmarking/base/core/univariate/TemporalDisaggregationResults;",
            method = "processDisaggregation",
            jseries, constant, trend, jindicators, model, average, rho, rho.fixed,
            rho.truncated, zeroinitialization, diffuse.algorithm, diffuse.regressors
        )
    } else {
        jrslt <- .jcall(
            obj = "jdplus/benchmarking/base/r/TemporalDisaggregation",
            returnSig = "Ljdplus/benchmarking/base/core/univariate/TemporalDisaggregationResults;",
            method = "processDisaggregation",
            jseries, constant, trend, model, as.integer(freq), average, rho, rho.fixed,
            rho.truncated, zeroinitialization, diffuse.algorithm, diffuse.regressors,
            as.integer(nbcsts), as.integer(nfcsts)
        )
    }


    # Build the S3 result
    bcov <- rjd3toolkit::.proc_matrix(jrslt, "covar")
    vars <- rjd3toolkit::.proc_vector(jrslt, "regnames")
    coef <- rjd3toolkit::.proc_vector(jrslt, "coeff")
    se <- sqrt(diag(bcov))
    t <- coef / se
    m <- data.frame(coef, se, t)
    row.names(m) <- vars

    regression <- list(
        type = model,
        conversion = ifelse(average, "Average", "Sum"),
        model = m,
        cov = bcov
    )

    disagg <- rjd3toolkit::.proc_ts(jrslt, "disagg")
    estimation <- list(
        disagg = disagg,
        edisagg = rjd3toolkit::.proc_ts(jrslt, "edisagg"),
        regeffect = rjd3toolkit::.proc_ts(jrslt, "regeffect"),
        smoothingpart = rjd3toolkit::.proc_numeric(jrslt, "smoothingpart"),
        parameter = rjd3toolkit::.proc_numeric(jrslt, "parameter"),
        eparameter = rjd3toolkit::.proc_numeric(jrslt, "eparameter"),
        residuals = .proc_residuals(jrslt, stats::frequency(disagg))
    )
    likelihood <- rjd3toolkit::.proc_likelihood(jrslt, "likelihood.")

    output <- list(
        regression = regression,
        estimation = estimation,
        likelihood = likelihood
    )
    class(output) <- "JD3_TEMPDISAGG_RSLTS"
    return(output)
}


#' @title Temporal Disaggregation of an Atypical Frequency Series by Regression Models.
#'
#' @description
#' Perform temporal disaggregation of low-frequency to high-frequency time
#' series by regression models. The implemented models include Chow-Lin,
#' Fernandez, Litterman and some variants of those algorithms. The
#' `temporal_disaggregation_raw()` function extends `temporal_disaggregation()`
#' by allowing temporal disaggregation for any frequency ratio.
#'
#' @param series A low-frequency time series to be disaggregated. It must be a numeric vector.
#' @param constant Boolean. Indicates whether a constant term is included in the model. The default is `TRUE`.
#' Note that this argument is used only with `model = "Ar1"` when `zeroinitialization = FALSE`. For additional information on this, see the package vignette.
#' @param trend Boolean. Indicates whether a linear trend is included in the model. The default is `FALSE`.
#' @param indicators One or more high-frequency indicator series used in the temporal disaggregation.
#' If `NULL` (the default), no indicator is used. When provided, the argument must be a numeric vector or a matrix.
#' @param startoffset The number of initial observations in the indicator series that precede the start of the low-frequency series.
#' The value must be either 0 or a positive integer (default is 0). This argument is ignored when no indicator is provided.
#' @param model A character string specifying the model of the error term at the disaggregated level.
#' The options are: `"Ar1"` (Chow Lin, the default), `"Rw"` (Fernandez), and `"RwAr1"` (Litterman).
#' @param freqratio An integer specifying the frequency ratio between the disaggregated series and the low-frequency series.
#' This argument is mandatory and must be a positive integer.
#' @param average Boolean. Indicates whether an average conversion should be considered. The default is `FALSE`, corresponding to additive conversion.
#' @param rho A numeric value giving the (initial) value of the autoregressive parameter.
#' This argument is used only for `"Ar1"` and `"RwAr1"` models.
#' @param rho.fixed Boolean. Specifies whether the supplied value of `rho` is fixed. The default is `FALSE`, which indicates that `rho` is estimated.
#' @param rho.truncated A numeric value defining the lower bound of the admissible range for `rho`.
#' The evaluation range is `[rho.truncated, 1[`.
#' @param zeroinitialization Boolean. If `TRUE`, the initial values of the autoregressive model are set to zero. The default is `FALSE`.
#' @param diffuse.algorithm A character string specifying the algorithm used for diffuse initialization. The default is `"SqrtDiffuse"`. Other options are: `"Diffuse"` and `"Augmented"`.
#' @param diffuse.regressors Boolean. Indicates whether the coefficients of the regression model are treated as diffuse (`TRUE`) or as fixed unknown (`FALSE`, the default).
#' @param nbcsts An integer specifying the number of backcast periods.
#' This argument is ignored when one or more indicator series is provided.
#' @param nfcsts An integer specifying the number of forecast periods.
#' This argument is ignored when one or more indicator series is provided.
#'
#' @return An object of class "JD3_TEMPDISAGGRAW_RSLTS" is returned. The following are returned
#' invisibly as a list:
#' * `regression` `[[1]]` regression coefficients;
#' * `estimation` `[[2]]` disaggregated values and standard deviation, regression effects, smoothing part, parameter and residuals;
#' * `likelihood` `[[3]]` likelihood statistics.
#'
#' @export
#'
#' @seealso `temporal_interpolation_raw()`
#'
#' For more information, see the vignette:
#'
#' `utils::browseVignettes()`, e.g. `browseVignettes(package = "rjd3bench")`
#'
#' @examplesIf rjd3toolkit::get_java_version() >= rjd3toolkit::minimal_java_version
#' # Use of Chow-lin method to disaggregate a biennial series with an annual indicator
#' Y <- stats::aggregate(rjd3toolkit::Retail$RetailSalesTotal, 0.5)
#' x <- stats::aggregate(rjd3toolkit::Retail$FoodAndBeverageStores, 1)
#' td <- temporal_disaggregation_raw(as.numeric(Y), indicators = as.numeric(x), freqratio = 2)
#' td$estimation$disagg
#'
#' # Use of Fernandez method to disaggregate a series without indicator
#' # considering a frequency ratio of 5 (for example, it could be a quinquennial
#' # series to disaggregate on an annual basis)
#' Y2 <- c(500,510,525,520)
#' td2 <- temporal_disaggregation_raw(Y2, model = "Rw", freqratio = 5, nfcsts = 2)
#' td2$estimation$disagg
#'
#' # Same with an indicator, considering an offset in the latter
#' Y2 <- c(500,510,525,520)
#' x2 <- c(97,
#'         98, 98.5, 99.5, 104, 99,
#'         100, 100.5, 101, 105.5, 103,
#'         104.5, 103.5, 104.5, 109, 104,
#'         107, 103, 108, 113, 110)
#' td3 <- temporal_disaggregation_raw(Y2, indicators = x2, startoffset = 1,
#'                                    model = "Rw", freqratio = 5)
#' td3$estimation$disagg
#'
temporal_disaggregation_raw <- function(
        series,
        constant = TRUE,
        trend = FALSE,
        indicators = NULL,
        startoffset = 0L,
        model = c("Ar1", "Rw", "RwAr1"),
        freqratio,
        average = FALSE,
        rho = 0.0,
        rho.fixed = FALSE,
        rho.truncated = 0.0,
        zeroinitialization = FALSE,
        diffuse.algorithm = c("SqrtDiffuse", "Diffuse", "Augmented"),
        diffuse.regressors = FALSE,
        nbcsts = 0L,
        nfcsts = 0L) {

    model <- match.arg(model)
    diffuse.algorithm <- match.arg(diffuse.algorithm)
    if(!is.vector(series, mode = "numeric")){
        stop("The input series must be a numeric vector")
    }
    if (model != "Ar1" && !zeroinitialization) {
        constant <- FALSE
    }

    if (!is.null(indicators)) {
        if (is.matrix(indicators)) {
            jindicators <- rjd3toolkit::.r2jd_matrix(indicators)
        } else if (is.vector(indicators, mode = "numeric")) {
            jindicators <- rjd3toolkit::.r2jd_matrix(as.matrix(indicators))
        } else{
            stop("Indicators must be either a numeric vector or a matrix")
        }
        jrslt <- .jcall(
            obj = "jdplus/benchmarking/base/r/TemporalDisaggregation",
            returnSig = "Ljdplus/benchmarking/base/core/univariate/RawTemporalDisaggregationResults;",
            method = "processRawDisaggregation",
            as.numeric(series), constant, trend, jindicators, as.integer(startoffset), model,
            as.integer(freqratio), average, rho, rho.fixed,
            rho.truncated, zeroinitialization, diffuse.algorithm, diffuse.regressors
        )
    } else{
        jrslt <- .jcall(
            obj = "jdplus/benchmarking/base/r/TemporalDisaggregation",
            returnSig = "Ljdplus/benchmarking/base/core/univariate/RawTemporalDisaggregationResults;",
            method = "processRawDisaggregation",
            as.numeric(series), constant, trend, model, as.integer(freqratio), average,
            rho, rho.fixed, rho.truncated, zeroinitialization, diffuse.algorithm, diffuse.regressors,
            as.integer(nbcsts), as.integer(nfcsts)
        )
    }

    # Build the S3 result
    bcov <- rjd3toolkit::.proc_matrix(jrslt, "covar")
    vars <- c()
    if(constant) vars <- "C"
    if(trend) vars <- c(vars, "Trend")
    if (!is.null(indicators)) {
        if (is.matrix(indicators)) {
            for (i in 1:ncol(indicators)) {
                vars <- c(vars, paste0("var", i))
            }
        }
        else vars <- c(vars, "var1")
    }
    coef <- rjd3toolkit::.proc_vector(jrslt, "coeff")
    se <- sqrt(diag(bcov))
    t <- coef/se
    m <- data.frame(coef, se, t)
    if(nrow(m) > 0) row.names(m) <- vars

    regression <- list(
        type = model,
        conversion = ifelse(average, "Average", "Sum"),
        model = m,
        cov = bcov
    )

    estimation <- list(
        disagg = rjd3toolkit::.proc_vector(jrslt, "disagg"),
        edisagg = rjd3toolkit::.proc_vector(jrslt, "edisagg"),
        regeffect = rjd3toolkit::.proc_vector(jrslt, "regeffect"),
        smoothingpart = ifelse(!is.null(vars), rjd3toolkit::.proc_numeric(jrslt, "smoothingpart"), NaN),
        parameter = rjd3toolkit::.proc_numeric(jrslt, "parameter"),
        eparameter = rjd3toolkit::.proc_numeric(jrslt, "eparameter"),
        residuals = .proc_residuals(jrslt, freqratio)
    )
    likelihood <- rjd3toolkit::.proc_likelihood(jrslt, "likelihood.")

    output <- list(
        regression = regression,
        estimation = estimation,
        likelihood = likelihood
    )
    class(output) <- "JD3_TEMPDISAGGRAW_RSLTS"
    return(output)
}


#' @title Interpolation of a Time Series by Regression Models.
#'
#' @description
#' Perform temporal interpolation of low-frequency to high-frequency time
#' series by regression models. The implemented models include Chow-Lin,
#' Fernandez, Litterman and some variants of those algorithms.
#'
#' @param series A low-frequency time series to be interpolated. It must be a `"ts"` object.
#' @param constant Boolean. Indicates whether a constant term is included in the model. The default is `TRUE`.
#' Note that this argument is used only with `model = "Ar1"` when `zeroinitialization = FALSE`. For additional information, see the package vignette.
#' @param trend Boolean. Indicates whether a linear trend is included in the model. The default is `FALSE`.
#' @param indicators One or more high-frequency indicator series used in the temporal interpolation.
#' If `NULL` (the default), no indicator is used. When provided, the argument must be a `"ts"` object or a list of `"ts"` objects.
#' @param model A character string specifying the model of the error term at the interpolated level.
#' The options are: `"Ar1"` (Chow Lin, the default), `"Rw"` (Fernandez), and `"RwAr1"` (Litterman).
#' @param freq An integer giving the annual frequency of the interpolated series.
#' This argument is ignored when one or more indicator series is provided.
#' @param obsposition An integer specifying the position of the low-frequency
#'   observations within the interpolated series (e.g. the 1st month of the
#'   year, the 2d month, etc.). The value must be a positive integer or `-1` (the
#'   default). The default value is equivalent to setting the value of the
#'   parameter equal to the frequency of the series, meaning that the last value
#'   of the interpolated series is consistent with the low-frequency series.
#' @param rho A numeric value giving the (initial) value of the autoregressive parameter.
#' This argument is used only for `"Ar1"` and `"RwAr1"` models.
#' @param rho.fixed Boolean. Specifies whether the supplied value of `rho` is fixed. The default is `FALSE`, which indicates that `rho` is estimated.
#' @param rho.truncated A numeric value defining the lower bound of the admissible range for `rho`.
#' The evaluation range is `[rho.truncated, 1[`.
#' @param zeroinitialization Boolean. If `TRUE`, the initial values of the autoregressive model are set to zero. The default is `FALSE`.
#' @param diffuse.algorithm A character string specifying the algorithm used for diffuse initialization. The default is `"SqrtDiffuse"`. Other options are: `"Diffuse"` and `"Augmented"`.
#' @param diffuse.regressors Boolean. Indicates whether the coefficients of the regression model are treated as diffuse (`TRUE`) or as fixed unknown (`FALSE`, the default).
#' @param nbcsts An integer specifying the number of backcast periods.
#' This argument is ignored when one or more indicator series is provided.
#' @param nfcsts An integer specifying the number of forecast periods.
#' This argument is ignored when one or more indicator series is provided.
#'
#' @return An object of class "JD3_INTERP_RSLTS" is returned. The following are returned
#' invisibly as a list:
#' * `regression` `[[1]]` regression coefficients;
#' * `estimation` `[[2]]` interpolated Time-Series and standard deviation, regression effects and smoothing part, parameter and residuals;
#' * `likelihood` `[[3]]` likelihood statistics.
#'
#' @export
#'
#' @seealso `temporal_disaggregation()`,
#'
#' `temporal_interpolation_raw()` for interpolation of atypical frequency series,
#'
#' `temporal_disaggregation_raw()` for temporal disaggregation of atypical frequency series
#'
#'
#' For more information, see the vignette:
#'
#' `utils::browseVignettes()`, e.g. `browseVignettes(package = "rjd3bench")`
#'
#' @examplesIf rjd3toolkit::get_java_version() >= rjd3toolkit::minimal_java_version
#' # Chow-lin / Fernandez when the last value of the interpolated series is
#' # consistent with the low-frequency series
#' Y <- rjd3toolkit::aggregate(rjd3toolkit::Retail$RetailSalesTotal, 1)
#' x <- rjd3toolkit::Retail$FoodAndBeverageStores
#' ti1 <- temporal_interpolation(Y, indicators = x)
#' ti1$estimation$interp
#'
#' ti2 <- temporal_interpolation(Y, indicators = x, model = "Rw")
#' ti2$estimation$interp
#'
#' # Same without indicator
#' ti3 <- temporal_interpolation(Y, model = "Rw", freq = 12, nfcsts = 6)
#' ti3$estimation$interp
#'
#  # Chow-lin when the first value of the interpolated series is the one
#  # consistent with the low-frequency series
#' ti4 <- temporal_interpolation(Y, indicators = x, obsposition = 1)
#' ti4$estimation$interp
#'
temporal_interpolation <- function(
        series,
        constant = TRUE,
        trend = FALSE,
        indicators = NULL,
        model = c("Ar1", "Rw", "RwAr1"),
        freq = 4L,
        obsposition = -1L,
        rho = 0.0,
        rho.fixed = FALSE,
        rho.truncated = 0.0,
        zeroinitialization = FALSE,
        diffuse.algorithm = c("SqrtDiffuse", "Diffuse", "Augmented"),
        diffuse.regressors = FALSE,
        nbcsts = 0L,
        nfcsts = 0L) {

    model <- match.arg(model)
    diffuse.algorithm <- match.arg(diffuse.algorithm)
    if (model != "Ar1" && !zeroinitialization) {
        constant <- FALSE
    }
    if(obsposition > 0){
        obsposition <- obsposition - 1L
    }else if (obsposition != -1){
        stop("obsposition must be set to -1 (default) or a positive integer")
    }

    jseries <- rjd3toolkit::.r2jd_tsdata(series)
    if (!is.null(indicators)) {
        jlist <- list()
        if (is.list(indicators)) {
            for (i in seq_along(indicators)) {
                jlist[[i]] <- rjd3toolkit::.r2jd_tsdata(indicators[[i]])
            }
        } else if (stats::is.ts(indicators)) {
            jlist[[1L]] <- rjd3toolkit::.r2jd_tsdata(indicators)
        } else {
            stop("Invalid indicators")
        }
        jindicators <- .jarray(jlist, contents.class = "jdplus/toolkit/base/api/timeseries/TsData")

        jrslt <- .jcall(
            obj = "jdplus/benchmarking/base/r/TemporalDisaggregation",
            returnSig = "Ljdplus/benchmarking/base/core/univariate/TemporalDisaggregationResults;",
            method = "processInterpolation",
            jseries, constant, trend, jindicators, model, as.integer(obsposition), rho, rho.fixed,
            rho.truncated, zeroinitialization, diffuse.algorithm, diffuse.regressors
        )
    } else {
        jrslt <- .jcall(
            obj = "jdplus/benchmarking/base/r/TemporalDisaggregation",
            returnSig = "Ljdplus/benchmarking/base/core/univariate/TemporalDisaggregationResults;",
            method = "processInterpolation",
            jseries, constant, trend, model, as.integer(freq), as.integer(obsposition), rho, rho.fixed,
            rho.truncated, zeroinitialization, diffuse.algorithm, diffuse.regressors,
            as.integer(nbcsts), as.integer(nfcsts)
        )
    }


    # Build the S3 result
    bcov <- rjd3toolkit::.proc_matrix(jrslt, "covar")
    vars <- rjd3toolkit::.proc_vector(jrslt, "regnames")
    coef <- rjd3toolkit::.proc_vector(jrslt, "coeff")
    se <- sqrt(diag(bcov))
    t <- coef / se
    m <- data.frame(coef, se, t)
    row.names(m) <- vars

    interp <- rjd3toolkit::.proc_ts(jrslt, "disagg")
    f <- stats::frequency(interp)

    regression <- list(
        type = model,
        obsposition = ifelse(obsposition == -1L, f, obsposition),
        model = m,
        cov = bcov
    )

    estimation <- list(
        interp = interp,
        einterp = rjd3toolkit::.proc_ts(jrslt, "edisagg"),
        regeffect = rjd3toolkit::.proc_ts(jrslt, "regeffect"),
        smoothingpart = rjd3toolkit::.proc_numeric(jrslt, "smoothingpart"),
        parameter = rjd3toolkit::.proc_numeric(jrslt, "parameter"),
        eparameter = rjd3toolkit::.proc_numeric(jrslt, "eparameter"),
        residuals = .proc_residuals(jrslt, f)
    )
    likelihood <- rjd3toolkit::.proc_likelihood(jrslt, "likelihood.")

    output <- list(
        regression = regression,
        estimation = estimation,
        likelihood = likelihood
    )
    class(output) <- "JD3_INTERP_RSLTS"
    return(output)
}


#' @title Interpolation of an Atypical Frequency Series by Regression Models.
#'
#' @description
#' Perform temporal interpolation of low-frequency to high-frequency time series
#' by regression models. The implemented models include Chow-Lin, Fernandez,
#' Litterman and some variants of those algorithms. The
#' `temporal_interpolation_raw()` function extends `temporal_interpolation()` by
#' allowing temporal interpolation for any frequency ratio.
#'
#' @param series A low-frequency time series to be interpolated. It must be a numeric vector.
#' @param constant Boolean. Indicates whether a constant term is included in the model. The default is `TRUE`.
#' Note that this argument is used only with `model = "Ar1"` when `zeroinitialization = FALSE`. For additional information, see the package vignette.
#' @param trend Boolean. Indicates whether a linear trend is included in the model. The default is `FALSE`.
#' @param indicators One or more high‑frequency indicator series used in the interpolation.
#' If `NULL` (the default), no indicator is used. When provided, the argument must be a numeric vector or a matrix.
#' @param startoffset The number of initial observations in the indicator series that precede the start of the low-frequency series.
#' The value must be either 0 or a positive integer (default is 0). This argument is ignored when no indicator is provided.
#' @param model A character string specifying the model of the error term at the disaggregated level.
#' The options are: `"Ar1"` (Chow Lin, the default), `"Rw"` (Fernandez), and `"RwAr1"` (Litterman).
#' @param freqratio An integer specifying the frequency ratio between the interpolated series and the low-frequency series.
#' This argument is mandatory and must be a positive integer.
#' @param obsposition An integer specifying the position of the low-frequency
#'   observations within the interpolated series (e.g. the 1st month of the
#'   year, the 2d month, etc.). The value must be a positive integer or `-1`
#'   (the default).The default value is equivalent to setting the value of the
#'   parameter equal to the frequency of the series, meaning that the last value
#'   of the interpolated series is consistent with the low-frequency series.
#' @param rho A numeric value giving the (initial) value of the autoregressive parameter.
#' This argument is used only for `"Ar1"` and `"RwAr1"` models.
#' @param rho.fixed Boolean. Specifies whether the supplied value of `rho` is fixed. The default is `FALSE`, which indicates that `rho` is estimated.
#' @param rho.truncated A numeric value defining the lower bound of the admissible range for `rho`.
#' The evaluation range is `[rho.truncated, 1[`.
#' @param zeroinitialization Boolean. If `TRUE`, the initial values of the autoregressive model are set to zero. The default is `FALSE`.
#' @param diffuse.algorithm A character string specifying the algorithm used for diffuse initialization. The default is `"SqrtDiffuse"`. Other options are: `"Diffuse"` and `"Augmented"`.
#' @param diffuse.regressors Boolean. Indicates whether the coefficients of the regression model are treated as diffuse (`TRUE`) or as fixed unknown (`FALSE`, the default).
#' @param nbcsts An integer specifying the number of backcast periods.
#' This argument is ignored when one or more indicator series is provided.
#' @param nfcsts An integer specifying the number of forecast periods.
#' This argument is ignored when one or more indicator series is provided.
#'
#' @return An object of class "JD3_INTERPRAW_RSLTS" is returned. The following are returned
#' invisibly as a list:
#' * `regression` `[[1]]` regression coefficients;
#' * `estimation` `[[2]]` interpolated values and standard deviation, regression effects, smoothing part, parameter and residuals;
#' * `likelihood` `[[3]]` likelihood statistics.
#'
#' @export
#'
#' @seealso `temporal_disaggregation_raw()`
#'
#' For more information, see the vignette:
#'
#' `utils::browseVignettes()`, e.g. `browseVignettes(package = "rjd3bench")`
#'
#' @examplesIf rjd3toolkit::get_java_version() >= rjd3toolkit::minimal_java_version
#' # Use of Chow-lin method to interpolate a biennial series with an annual
#' # indicator (the low-frequency series is consistent with the last value of the
#' # interpolated series)
#' Y <- stats::aggregate(rjd3toolkit::Retail$RetailSalesTotal, 0.5)
#' x <- stats::aggregate(rjd3toolkit::Retail$FoodAndBeverageStores, 1)
#' ti <- temporal_interpolation_raw(as.numeric(Y), indicators = as.numeric(x), freqratio = 2)
#' ti$estimation$interp
#'
#' # Use of Fernandez method to interpolate a series without indicator
#' # considering a frequency ratio of 5 (the low-frequency series is consistent
#' # with the last value of the interpolated series). For example, Y2 could be a
#' # quinquennial series to interpolate annually.
#' Y2 <- c(500,510,525,520)
#' ti2 <- temporal_interpolation_raw(Y2, model = "Rw", freqratio = 5, nbcsts = 1, nfcsts = 2)
#' ti2$estimation$interp
#'
#' # Same with an indicator, considering an offset in the latter
#' Y2 <- c(500,510,525,520)
#' x2 <- c(485,
#'         490, 492.5, 497.5, 520, 495,
#'         500, 502.5, 505, 527.5, 515,
#'         522.5, 517.5, 522.5, 545, 520,
#'         535, 515, 540, 565, 550)
#' ti3 <- temporal_interpolation_raw(Y2, indicators = x2, startoffset = 1, model = "Rw", freqratio = 5)
#' ti3$estimation$interp
#'
#' # Same considering that the first value of the interpolated series is the one
#' # consistent with the low-frequency series
#' ti4 <- temporal_interpolation_raw(Y2, indicators = x2, startoffset = 1,
#'                                   model = "Rw", freqratio = 5, obsposition = 1)
#' ti4$estimation$interp
#'
temporal_interpolation_raw <- function(
        series,
        constant = TRUE,
        trend = FALSE,
        indicators = NULL,
        startoffset = 0L,
        model = c("Ar1", "Rw", "RwAr1"),
        freqratio,
        obsposition = -1L,
        rho = 0.0,
        rho.fixed = FALSE,
        rho.truncated = 0.0,
        zeroinitialization = FALSE,
        diffuse.algorithm = c("SqrtDiffuse", "Diffuse", "Augmented"),
        diffuse.regressors = FALSE,
        nbcsts = 0L,
        nfcsts = 0L) {

    model <- match.arg(model)
    diffuse.algorithm <- match.arg(diffuse.algorithm)
    if(!is.vector(series, mode = "numeric")){
        stop("The input series must be a numeric vector")
    }
    if (model != "Ar1" && !zeroinitialization) {
        constant <- FALSE
    }
    if(obsposition > 0){
        obsposition <- obsposition - 1L
    }else if (obsposition != -1){
        stop("obsposition must be set to -1 (default) or a positive integer")
    }

    if (!is.null(indicators)) {
        if (is.matrix(indicators)) {
            jindicators <- rjd3toolkit::.r2jd_matrix(indicators)
        } else if (is.vector(indicators, mode = "numeric")) {
            jindicators <- rjd3toolkit::.r2jd_matrix(as.matrix(indicators))
        } else{
            stop("Indicators must be either a numeric vector or a matrix")
        }
        jrslt <- .jcall(
            obj = "jdplus/benchmarking/base/r/TemporalDisaggregation",
            returnSig = "Ljdplus/benchmarking/base/core/univariate/RawTemporalDisaggregationResults;",
            method = "processRawInterpolation",
            as.numeric(series), constant, trend, jindicators, as.integer(startoffset), model,
            as.integer(freqratio), as.integer(obsposition), rho, rho.fixed,
            rho.truncated, zeroinitialization, diffuse.algorithm, diffuse.regressors
        )
    } else{
        jrslt <- .jcall(
            obj = "jdplus/benchmarking/base/r/TemporalDisaggregation",
            returnSig = "Ljdplus/benchmarking/base/core/univariate/RawTemporalDisaggregationResults;",
            method = "processRawInterpolation",
            as.numeric(series), constant, trend, model, as.integer(freqratio), as.integer(obsposition),
            rho, rho.fixed, rho.truncated, zeroinitialization, diffuse.algorithm, diffuse.regressors,
            as.integer(nbcsts), as.integer(nfcsts)
        )
    }

    # Build the S3 result
    bcov <- rjd3toolkit::.proc_matrix(jrslt, "covar")
    vars <- c()
    if(constant) vars <- "C"
    if(trend) vars <- c(vars, "Trend")
    if (!is.null(indicators)) {
        if (is.matrix(indicators)) {
            for (i in 1:ncol(indicators)) {
                vars <- c(vars, paste0("var", i))
            }
        }
        else vars <- c(vars, "var1")
    }
    coef <- rjd3toolkit::.proc_vector(jrslt, "coeff")
    se <- sqrt(diag(bcov))
    t <- coef/se
    m <- data.frame(coef, se, t)
    if(nrow(m) > 0) row.names(m) <- vars

    regression <- list(
        type = model,
        obsposition = ifelse(obsposition == -1L, freqratio, obsposition),
        model = m,
        cov = bcov
    )

    estimation <- list(
        interp = rjd3toolkit::.proc_vector(jrslt, "disagg"),
        einterp = rjd3toolkit::.proc_vector(jrslt, "edisagg"),
        regeffect = rjd3toolkit::.proc_vector(jrslt, "regeffect"),
        smoothingpart = ifelse(!is.null(vars), rjd3toolkit::.proc_numeric(jrslt, "smoothingpart"), NaN),
        parameter = rjd3toolkit::.proc_numeric(jrslt, "parameter"),
        eparameter = rjd3toolkit::.proc_numeric(jrslt, "eparameter"),
        residuals = .proc_residuals(jrslt, freqratio)
    )
    likelihood <- rjd3toolkit::.proc_likelihood(jrslt, "likelihood.")

    output <- list(
        regression = regression,
        estimation = estimation,
        likelihood = likelihood
    )
    class(output) <- "JD3_INTERPRAW_RSLTS"
    return(output)
}


#' @title Temporal Disaggregation and Interpolation of a Time Series by means of a Reverse Regression Model.
#'
#' @description
#' Perform temporal disaggregation and interpolation of low-frequency to high
#' frequency time series by means of a reverse regression model. Unlike the
#' usual regression-based models, this approach treats a high-frequency
#' indicator as the dependent variable and the unknown target series as the
#' independent variable.
#'
#' @param series A low-frequency time series to be disaggregated or interpolated. It must be a `"ts"` object.
#' @param indicator A high-frequency indicator series. It must be a `"ts"` object.
#' @param conversion A character string specifying the conversion mode, typically `"Sum"`(the default) or `"Average"`. Other options are: `"Last"`, `"First"` and `"UserDefined"`.
#' @param conversion.obsposition An integer specifying the position of the low-frequency observations within the interpolated series (e.g. the 7th month of the year).
#' This argument is used only for interpolation when `conversion = "UserDefined"`.
#' @param rho A numeric value giving the (initial) value of the autoregressive parameter.
#' @param rho.fixed Boolean. Specifies whether the supplied value of `rho` is fixed. The default is `FALSE`, which indicates that `rho` is estimated.
#' @param rho.truncated A numeric value defining the lower bound of the admissible range for `rho`.
#' The evaluation range is `[rho.truncated, 1[`.
#'
#' @return An object of class "JD3_TEMPDISAGGI_RSLTS" is returned. The following are returned
#' invisibly as a list:
#' * `regression` `[[1]]` regression coefficients;
#' * `estimation` `[[2]]` disaggregated Time-Series and parameter;
#' * `likelihood` `[[3]]` likelihood statistics.
#'
#' @references  Bournay J., Laroque G. (1979). Reflexions sur la methode
#'   d'elaboration des comptes trimestriels. Annales de l'Insee, n. 36, pp.3-30.
#'
#' @export
#'
#' @seealso For more information, see the vignette:
#'
#' `utils::browseVignettes()`, e.g. `browseVignettes(package = "rjd3bench")`
#'
#' @examplesIf rjd3toolkit::get_java_version() >= rjd3toolkit::minimal_java_version
#' # Retail data, monthly indicator
#' Y <- rjd3toolkit::aggregate(rjd3toolkit::Retail$RetailSalesTotal, 1)
#' x <- rjd3toolkit::Retail$FoodAndBeverageStores
#' td <- temporaldisaggregationI(Y, indicator = x)
#' td$estimation$disagg
#'
#' # qna data, quarterly indicator
#' data("qna_data")
#' Y <- ts(qna_data$B1G_Y_data[,"B1G_CE"], frequency = 1, start = c(2009,1))
#' x <- ts(qna_data$TURN_Q_data[,"TURN_INDEX_CE"], frequency = 4, start = c(2009,1))
#' td <- temporaldisaggregationI(Y, indicator = x)
#' td$regression$a
#' td$regression$b
#'
temporaldisaggregationI <- function(series, indicator,
                                    conversion = c("Sum", "Average", "Last", "First", "UserDefined"), conversion.obsposition = 1L,
                                    rho = 0., rho.fixed = FALSE,  rho.truncated = 0.) {
    conversion <- match.arg(conversion)
    jseries <- rjd3toolkit::.r2jd_tsdata(series)
    jlist <- list()
    jindicator <- rjd3toolkit::.r2jd_tsdata(indicator)
    jrslt <- .jcall("jdplus/benchmarking/base/r/TemporalDisaggregation", "Ljdplus/benchmarking/base/core/univariate/TemporalDisaggregationIResults;",
                    "processI", jseries, jindicator, "Ar1", conversion, as.integer(conversion.obsposition), rho, rho.fixed, rho.truncated)
    # Build the S3 result
    a <- rjd3toolkit::.proc_numeric(jrslt, "a")
    b <- rjd3toolkit::.proc_numeric(jrslt, "b")

    regression <- list(
        conversion = conversion,
        a = a,
        b = b
    )
    estimation <- list(
        disagg = rjd3toolkit::.proc_ts(jrslt, "disagg"),
        parameter = rjd3toolkit::.proc_numeric(jrslt, "parameter")
    )
    likelihood <- rjd3toolkit::.proc_likelihood(jrslt, "likelihood.")

    output <- list(
        regression = regression,
        estimation = estimation,
        likelihood = likelihood
    )
    class(output) <- "JD3_TEMPDISAGGI_RSLTS"
    return(output)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tempdisagg.R
\name{temporal_interpolation}
\alias{temporal_interpolation}
\title{Interpolation of a Time Series by Regression Models.}
\usage{
temporal_interpolation(
  series,
  constant = TRUE,
  trend = FALSE,
  indicators = NULL,
  model = c("Ar1", "Rw", "RwAr1"),
  freq = 4L,
  obsposition = -1L,
  rho = 0,
  rho.fixed = FALSE,
  rho.truncated = 0,
  zeroinitialization = FALSE,
  diffuse.algorithm = c("SqrtDiffuse", "Diffuse", "Augmented"),
  diffuse.regressors = FALSE,
  nbcsts = 0L,
  nfcsts = 0L
)
}
\arguments{
\item{series}{A low-frequency time series to be interpolated. It must be a \code{"ts"} object.}

\item{constant}{Boolean. Indicates whether a constant term is included in the model. The default is \code{TRUE}.
Note that this argument is used only with \code{model = "Ar1"} when \code{zeroinitialization = FALSE}. For additional information, see the package vignette.}

\item{trend}{Boolean. Indicates whether a linear trend is included in the model. The default is \code{FALSE}.}

\item{indicators}{One or more high-frequency indicator series used in the temporal interpolation.
If \code{NULL} (the default), no indicator is used. When provided, the argument must be a \code{"ts"} object or a list of \code{"ts"} objects.}

\item{model}{A character string specifying the model of the error term at the interpolated level.
The options are: \code{"Ar1"} (Chow Lin, the default), \code{"Rw"} (Fernandez), and \code{"RwAr1"} (Litterman).}

\item{freq}{An integer giving the annual frequency of the interpolated series.
This argument is ignored when one or more indicator series is provided.}

\item{obsposition}{An integer specifying the position of the low-frequency
observations within the interpolated series (e.g. the 1st month of the
year, the 2d month, etc.). The value must be a positive integer or \code{-1} (the
default). The default value is equivalent to setting the value of the
parameter equal to the frequency of the series, meaning that the last value
of the interpolated series is consistent with the low-frequency series.}

\item{rho}{A numeric value giving the (initial) value of the autoregressive parameter.
This argument is used only for \code{"Ar1"} and \code{"RwAr1"} models.}

\item{rho.fixed}{Boolean. Specifies whether the supplied value of \code{rho} is fixed. The default is \code{FALSE}, which indicates that \code{rho} is estimated.}

\item{rho.truncated}{A numeric value defining the lower bound of the admissible range for \code{rho}.
The evaluation range is \verb{[rho.truncated, 1[}.}

\item{zeroinitialization}{Boolean. If \code{TRUE}, the initial values of the autoregressive model are set to zero. The default is \code{FALSE}.}

\item{diffuse.algorithm}{A character string specifying the algorithm used for diffuse initialization. The default is \code{"SqrtDiffuse"}. Other options are: \code{"Diffuse"} and \code{"Augmented"}.}

\item{diffuse.regressors}{Boolean. Indicates whether the coefficients of the regression model are treated as diffuse (\code{TRUE}) or as fixed unknown (\code{FALSE}, the default).}

\item{nbcsts}{An integer specifying the number of backcast periods.
This argument is ignored when one or more indicator series is provided.}

\item{nfcsts}{An integer specifying the number of forecast periods.
This argument is ignored when one or more indicator series is provided.}
}
\value{
An object of class "JD3_INTERP_RSLTS" is returned. The following are returned
invisibly as a list:
\itemize{
\item \code{regression} \verb{[[1]]} regression coefficients;
\item \code{estimation} \verb{[[2]]} interpolated Time-Series and standard deviation, regression effects and smoothing part, parameter and residuals;
\item \code{likelihood} \verb{[[3]]} likelihood statistics.
}
}
\description{
Perform temporal interpolation of low-frequency to high-frequency time
series by regression models. The implemented models include Chow-Lin,
Fernandez, Litterman and some variants of those algorithms.
}
\examples{
\dontshow{if (rjd3toolkit::get_java_version() >= rjd3toolkit::minimal_java_version) withAutoprint(\{ # examplesIf}
# Chow-lin / Fernandez when the last value of the interpolated series is
# consistent with the low-frequency series
Y <- rjd3toolkit::aggregate(rjd3toolkit::Retail$RetailSalesTotal, 1)
x <- rjd3toolkit::Retail$FoodAndBeverageStores
ti1 <- temporal_interpolation(Y, indicators = x)
ti1$estimation$interp

ti2 <- temporal_interpolation(Y, indicators = x, model = "Rw")
ti2$estimation$interp

# Same without indicator
ti3 <- temporal_interpolation(Y, model = "Rw", freq = 12, nfcsts = 6)
ti3$estimation$interp

ti4 <- temporal_interpolation(Y, indicators = x, obsposition = 1)
ti4$estimation$interp
\dontshow{\}) # examplesIf}
}
\seealso{
\code{temporal_disaggregation()},

\code{temporal_interpolation_raw()} for interpolation of atypical frequency series,

\code{temporal_disaggregation_raw()} for temporal disaggregation of atypical frequency series

For more information, see the vignette:

\code{utils::browseVignettes()}, e.g. \code{browseVignettes(package = "rjd3bench")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/benchmark.R
\name{denton_raw}
\alias{denton_raw}
\title{Benchmarking of an Atypical Frequency Series by means of the Denton Method.}
\usage{
denton_raw(
  s = NULL,
  t,
  freqratio,
  d = 1L,
  mul = TRUE,
  modified = TRUE,
  conversion = c("Sum", "Average", "Last", "First", "UserDefined"),
  obsposition = 1L,
  startoffset = 0L,
  nbcsts = 0L,
  nfcsts = 0L
)
}
\arguments{
\item{s}{A preliminary series. If not \code{NULL}, it must be a numeric vector.}

\item{t}{The low-frequency aggregation constraint. It must be a numeric vector.}

\item{freqratio}{An integer specifying the frequency ratio between the benchmarked series and the low-frequency constraint.
This argument is mandatory and must be a positive integer.}

\item{d}{An integer specifying the differencing order. The default is \code{1}.}

\item{mul}{Boolean. Indicates whether benchmarking is multiplicative (\code{TRUE}) or additive (\code{FALSE}). The default is multiplicative.}

\item{modified}{Boolean. Specifies whether the modified Denton method (\code{TRUE}) or the unmodified Denton method (\code{FALSE}) is applied. The default is \code{TRUE}.}

\item{conversion}{A character string specifying the conversion mode, typically \code{"Sum"} (the default) or \code{"Average"}. Other options are: \code{"Last"}, \code{"First"} and \code{"UserDefined"}.}

\item{obsposition}{An integer specifying the position of the observations of the low-frequency constraint within the benchmarked series (e.g. the 7th month of the year).
This argument is used only when \code{conversion = "UserDefined"}.}

\item{startoffset}{The number of initial observations in the preliminary series that precede the start of the low-frequency constraint.
The value must be either 0 or a positive integer (default is 0).
This argument is ignored when no preliminary series is provided.}

\item{nbcsts}{An integer specifying the number of backcast periods.
This argument is ignored when a preliminary series is provided.
(Not yet implemented.)}

\item{nfcsts}{An integer specifying the number of forecast periods.
This argument is ignored when a preliminary series is provided.
(Not yet implemented.)}
}
\value{
A numeric vector with the benchmarked series is returned.
}
\description{
Denton method relies on the principle of movement preservation. There exist a
few variants corresponding to different definitions of movement preservation:
additive first difference (AFD), proportional first difference (PFD),
additive second difference (ASD), proportional second difference (PSD), etc.
The default and most widely used is the Denton PFD method. The \code{denton_raw()}
function extends \code{denton()} by allowing benchmarking for any frequency ratio.
}
\examples{
\dontshow{if (rjd3toolkit::get_java_version() >= rjd3toolkit::minimal_java_version) withAutoprint(\{ # examplesIf}
Y <- c(500,510,525,520)
x <- c(97, 98, 98.5, 99.5, 104,
       99, 100, 100.5, 101, 105.5,
       103, 104.5, 103.5, 104.5, 109,
       104, 107, 103, 108, 113,
       110)

# Denton PFD
# for example, x and Y could be annual and quinquennal series respectively
denton_raw(x, Y, freqratio = 5)

# Denton AFD
denton_raw(x, Y, freqratio = 5, mul = FALSE)

# Denton PFD without indicator
denton_raw(t = Y, freqratio = 2, conversion = "Average")

# Denton PFD with/without an offset and conversion = "Last"
x2 <- c(485,
        490, 492.5, 497.5, 520, 495,
        500, 502.5, 505, 527.5, 515,
        522.5, 517.5, 522.5, 545, 520,
        535, 515, 540, 565, 550)
denton_raw(x2, Y, freqratio = 5, conversion = "Last")
denton_raw(x2, Y, freqratio = 5, conversion = "Last", startoffset = 1)

\dontshow{\}) # examplesIf}
}
\seealso{
For more information, see the vignette:

\code{utils::browseVignettes()}, e.g. \code{browseVignettes(package = "rjd3bench")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mbdenton.R
\name{denton_modelbased}
\alias{denton_modelbased}
\title{Temporal Disaggregation and Interpolation of a Time Series using the Model-Based Denton Proportional Method}
\usage{
denton_modelbased(
  series,
  indicator,
  differencing = 1L,
  conversion = c("Sum", "Average", "Last", "First", "UserDefined"),
  conversion.obsposition = 1L,
  outliers = NULL,
  fixedBIratios = NULL
)
}
\arguments{
\item{series}{A low-frequency time series to be disaggregated or interpolated. It must be either a \code{"ts"} object or a numeric vector.}

\item{indicator}{A high-frequency indicator series. It must be of the same class as \code{series}.}

\item{differencing}{Not yet implemented. This should be left equal to \code{1} (corresponding to the Denton PFD method).}

\item{conversion}{A character string specifying the conversion mode, typically \code{"Sum"} (the default) or \code{"Average"}. Other options are: \code{"Last"}, \code{"First"} and \code{"UserDefined"}.}

\item{conversion.obsposition}{An integer specifying the position of the low-frequency observations within the interpolated series (e.g. the 7th month of the year).
This argument is used only for interpolation when \code{conversion = "UserDefined"}.}

\item{outliers}{A list specifying the outlier periods and their magnitude.
Each element must be provided as \code{"YYYY-MM-DD" = value}, where the date
identifies the period. The numeric value specifies the intensity of the
outlier and corresponds to the relative value of the innovation variance
(with \code{1} indicating the normal situation).}

\item{fixedBIratios}{A list specifying the periods for which the
Benchmark‑to‑Indicator (BI) ratios should be fixed. Each element must be
provided as \code{"YYYY-MM-DD" = value}, where the date identifies the period
and the numeric value specifies the fixed BI ratio.}
}
\value{
An object of class "JD3_MBDENTON_RSLTS" is returned. The following are returned invisibly as a list:
\itemize{
\item \code{estimation} \verb{[[1]]} disaggregated Time-Series, BI ratios and standard deviations;
\item \code{likelihood} \verb{[[2]]} likelihood statistics.
}
}
\description{
The Denton proportional first difference (PFD) method can be expressed as a
statistical model in a state-space representation. This formulation provides
increased flexibility, including the ability to incorporate outliers, which
correspond to level shifts in the Benchmark‑to‑Indicator (BI) ratio, that
would otherwise induce unintended wave effects under the standard Denton PFD
method. In addition, the approach allows the disaggregated series to be
constrained (or 'frozen') at specific periods or prior to a given date by
fixing the corresponding high‑frequency BI ratios.
}
\examples{
\dontshow{if (rjd3toolkit::get_java_version() >= rjd3toolkit::minimal_java_version) withAutoprint(\{ # examplesIf}
# Retail data, monthly indicator
Y <- rjd3toolkit::aggregate(rjd3toolkit::Retail$RetailSalesTotal, 1)
x <- rjd3toolkit::aggregate(rjd3toolkit::Retail$FoodAndBeverageStores, 4)
td <- denton_modelbased(Y, x, outliers = list("2000-01-01" = 100, "2005-07-01" = 100))
y <- td$estimation$edisagg

# qna data, quarterly indicator
data("qna_data")
Y <- ts(qna_data$B1G_Y_data[,"B1G_FF"], frequency = 1, start = c(2009,1))
x <- ts(qna_data$TURN_Q_data[,"TURN_INDEX_FF"], frequency = 4, start = c(2009,1))

td1 <- denton_modelbased(Y, x)
td2 <- denton_modelbased(Y, x, outliers = list("2020-04-01" = 100),
                         fixedBIratios = list("2021-04-01" = 39.0))
bi1 <- td1$estimation$biratio
bi2 <- td2$estimation$biratio
y1 <- td1$estimation$disagg
y2 <- td2$estimation$disagg

stats::ts.plot(bi2, bi1, main = "BI ratios",
               gpars = list(col = c("red", "black")))
graphics::legend("topright", lty = 1, col = c("black", "red"),
                 legend = c("td1", "td2"))
stats::ts.plot(y2, y1, main = "Disaggregated series",
               gpars = list(col = c("red", "black")))
graphics::legend("topleft", lty = 1, col = c("black", "red"),
                 legend = c("td1", "td2"))
\dontshow{\}) # examplesIf}
}
\seealso{
For more information, see the vignette:

\code{utils::browseVignettes()}, e.g. \code{browseVignettes(package = "rjd3bench")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\docType{data}
\name{qna_data}
\alias{qna_data}
\title{Quarterly National Accounts data for temporal disaggregation}
\format{
A named list with two elements:
\describe{
\item{\code{B1G_Y_data}}{A data frame with columns:
\describe{
\item{\code{DATE}}{Annual periods.}
\item{\code{B1G_CE}}{Value added for chemical industry.}
\item{\code{B1G_FF}}{Value added for construction.}
\item{\code{B1G_HH}}{Value added for transport services.}
}
}
\item{\code{TURN_Q_data}}{A data frame with columns:
\describe{
\item{\code{DATE}}{Quarterly periods.}
\item{\code{TURN_INDEX_CE}}{Quarterly indicator for chemical industry.}
\item{\code{TURN_INDEX_FF}}{Quarterly indicator for construction.}
\item{\code{TURN_INDEX_HH}}{Quarterly indicator for transport services.}
}
}
}
}
\source{
Belgian Quarterly National Accounts
}
\usage{
qna_data
}
\description{
This dataset contains two data frames used for temporal disaggregation and
benchmarking exercises. The first data frame, \code{B1G_Y_data}, includes three
annual benchmark series corresponding to Belgian annual value added for the
period 2009–2020 in three industries: chemical industry (CE), construction
(FF), and transport services (HH). The second data frame, \code{TURN_Q_data},
contains the corresponding quarterly indicator series derived from VAT-based
production indicators, covering the period 2009Q1–2021Q4.
}
\examples{
data(qna_data)
names(qna_data)
head(qna_data$B1G_Y_data)
head(qna_data$TURN_Q_data)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/adl.R
\name{adl_disaggregation}
\alias{adl_disaggregation}
\title{Temporal Disaggregation of a Time Series by ADL Model}
\usage{
adl_disaggregation(
  series,
  constant = TRUE,
  trend = FALSE,
  indicators = NULL,
  average = FALSE,
  phi = 0,
  phi.fixed = FALSE,
  phi.truncated = 0,
  xar = c("FREE", "SAME", "NONE"),
  diffuse = FALSE
)
}
\arguments{
\item{series}{A low-frequency time series to be disaggregated. It must be \code{"ts"} object.}

\item{constant}{Boolean. Indicates whether a constant term is included in the model. The default is \code{TRUE}.}

\item{trend}{Boolean. Indicates whether a linear trend is included in the model. The default is \code{FALSE}.}

\item{indicators}{One or more high-frequency indicator series. If not NULL (the default), this must be a \code{"ts"} object or a list of \code{"ts"} objects.}

\item{average}{Boolean. Indicates whether an average conversion should be considered. The default is \code{FALSE}, corresponding to additive conversion.}

\item{phi}{A numeric value giving the (initial) value of the phi parameter}

\item{phi.fixed}{Boolean. Specifies whether the supplied value of \code{phi} is fixed. The default is \code{FALSE}, which indicates that \code{phi} is estimated.}

\item{phi.truncated}{A numeric value defining the lower bound of the admissible range for \code{phi}.
The evaluation range is \verb{[phi.truncated, 1[}.}

\item{xar}{A character string specifying the constraints imposed on the coefficients of the lagged regression variables. The default is \code{"FREE"}, which indicates that no constraints are applied. Other options are: \code{"SAME"}and \code{"NONE"}.
For additional information, see the package vignette.}

\item{diffuse}{Boolean. Indicates whether the coefficients of the regression model are treated as diffuse (\code{TRUE}) or as fixed unknown (\code{FALSE}, the default).}
}
\value{
An object of class "JD3_ADLDISAGG_RSLTS" is returned. The following are returned
invisibly as a list:
\itemize{
\item \code{regression} \verb{[[1]]} regression coefficients;
\item \code{estimation} \verb{[[2]]} disaggregated Time-Series and standard deviation, parameter and residuals;
\item \code{likelihood} \verb{[[3]]} likelihood statistics.
}
}
\description{
Perform temporal disaggregation of low-frequency to high-frequency time
series using an Autoregressive Distributed Lag regression model.
}
\examples{
\dontshow{if (rjd3toolkit::get_java_version() >= rjd3toolkit::minimal_java_version) withAutoprint(\{ # examplesIf}
# ADL model
data("qna_data")
Y <- ts(qna_data$B1G_Y_data[,"B1G_FF"], frequency = 1, start = c(2009,1))
x <- ts(qna_data$TURN_Q_data[,"TURN_INDEX_FF"], frequency = 4, start = c(2009,1))
td1 <- adl_disaggregation(Y, indicators = x, xar = "FREE")
td1$estimation$disagg

# ADL models with constraints
td2 <- adl_disaggregation(Y, indicators = x, xar = "SAME") # ~ Chow-Lin
td3 <- adl_disaggregation(Y, constant = FALSE, indicators = x,
                          xar = "SAME", phi = 1, phi.fixed = TRUE) # ~ Fernandez
td4 <- adl_disaggregation(Y, indicators = x, xar = "NONE") # ~ Santos Silva-Cardoso
\dontshow{\}) # examplesIf}
}
\references{
Proietti, P. (2005). Temporal Disaggregation by State Space Methods: Dynamic Regression Methods Revisited. Working papers and Studies, European Commission, ISSN 1725-4825.
}
\seealso{
For more information, see the vignette:

\code{utils::browseVignettes()}, e.g. \code{browseVignettes(package = "rjd3bench")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/benchmark.R
\name{cubicspline}
\alias{cubicspline}
\title{Benchmarking by means of Cubic Splines}
\usage{
cubicspline(
  s = NULL,
  t,
  nfreq = 4L,
  conversion = c("Sum", "Average", "Last", "First", "UserDefined"),
  obsposition = 1L
)
}
\arguments{
\item{s}{A preliminary series. If not \code{NULL}, it must be of the same class as \code{t}.}

\item{t}{The low-frequency aggregation constraint. It must be either an object of class \code{ts} or a numeric vector.}

\item{nfreq}{An integer giving the annual frequency of the benchmarked series.
This argument is used only when no preliminary series is provided.}

\item{conversion}{A character string specifying the conversion mode, typically \code{"Sum"} (the default) or \code{"Average"}. Other options are: \code{"Last"}, \code{"First"} and \code{"UserDefined"}.}

\item{obsposition}{An integer specifying the position of the observations of the low-frequency constraint within the benchmarked series (e.g. the 7th month of the year).
This argument is used only when \code{conversion = "UserDefined"}.}
}
\value{
A \code{"ts"} object with the benchmarked series is returned.
}
\description{
Cubic splines are piecewise cubic functions that are linked together in a way
to guarantee smoothness at data points. Additivity constraints are added for
benchmarking purpose and sub-period estimates are derived from each spline.
When a preliminary series is used, cubic splines are no longer drawn based on
the low-frequency constraint but the Benchmark-to-Indicator (BI ratio) is the
one being smoothed. Sub-period estimates are then simply the product between
the smoothed high frequency BI ratio and the preliminary series.
}
\examples{
\dontshow{if (rjd3toolkit::get_java_version() >= rjd3toolkit::minimal_java_version) withAutoprint(\{ # examplesIf}
data("qna_data")
Y <- ts(qna_data$B1G_Y_data[,"B1G_FF"], frequency = 1, start = c(2009,1))

# Cubic spline without preliminary series
y1 <- cubicspline(t = Y, nfreq = 4L)

# Cubic spline with preliminary series
x1 <- y1 + rnorm(n = length(y1), mean = 0, sd = 10)
cubicspline(s = x1, t = Y)

# Cubic splines used for temporal disaggregation
x2 <- ts(qna_data$TURN_Q_data[,"TURN_INDEX_FF"], frequency = 4, start = c(2009,1))
cubicspline(s = x2, t = Y)
\dontshow{\}) # examplesIf}
}
\seealso{
For more information, see the vignette:

\code{utils::browseVignettes()}, e.g. \code{browseVignettes(package = "rjd3bench")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/benchmark.R
\name{cholette}
\alias{cholette}
\title{Benchmarking by means of the Cholette Method}
\usage{
cholette(
  s,
  t,
  rho = 1,
  lambda = 1,
  bias = c("None", "Additive", "Multiplicative"),
  conversion = c("Sum", "Average", "Last", "First", "UserDefined"),
  obsposition = 1L
)
}
\arguments{
\item{s}{A preliminary series. It must be of the same class as \code{t}.}

\item{t}{The low-frequency aggregation constraint. It must be either a \code{"ts"} object or a numeric vector.}

\item{rho}{Numeric. A smoothing parameter whose value must lie between 0 and 1.
See the package vignette for more information on the choice of the \code{rho} parameter.}

\item{lambda}{Numeric. The adjustment model parameter. Typical choices include \code{lambda = 1} for proportional benchmarking, \code{lambda = 0} for additive benchmarking, and \code{lambda = 0.5} with \code{rho = 0} for the naive pro-rating method.
See the package vignette for more information on the choice of the \code{lambda} parameter.}

\item{bias}{Character. Specifies the bias-correction factor. By default, no systematic bias is considered. Other options are: "Additive" and "Multiplicative". See vignette for more details.
See the package vignette for more information on the other alternatives.}

\item{conversion}{A character string specifying the conversion mode, typically \code{"Sum"} (the default) or \code{"Average"}. Other options are: \code{"Last"}, \code{"First"} and \code{"UserDefined"}.}

\item{obsposition}{An integer specifying the position of the observations of the low-frequency constraint within the benchmarked series (e.g. the 7th month of the year).
This argument is used only when \code{conversion = "UserDefined"}.}
}
\value{
A \code{"ts"} object with the benchmarked series is returned.
}
\description{
Cholette method is based on a benchmarking methodology developed at
Statistics Canada. It is a generalized method relying on the principle of
movement preservation that encompasses other benchmarking methods.
The Denton method (both the AFD and PFD variants), as well as the naive
pro-rating method, emerge as particular cases of the Cholette method.
This method has been widely used for the purpose of benchmarking seasonally
adjusted series among others.
}
\examples{
\dontshow{if (rjd3toolkit::get_java_version() >= rjd3toolkit::minimal_java_version) withAutoprint(\{ # examplesIf}
ym_true <- rjd3toolkit::Retail$RetailSalesTotal
yq_true <- rjd3toolkit::aggregate(ym_true, 4)
Y_full <- rjd3toolkit::aggregate(ym_true, 1)

Y <- window(Y_full, end = c(2009,1)) # say no benchmark yet for the year 2010
xm <- ym_true + rnorm(n = length(ym_true), mean = -5000, sd = 10000)
xq <- rjd3toolkit::aggregate(xm, 4)

# Proportional benchmarking with a bias and some recommended value of rho for
# monthly and quarterly series respectively
cholette(s = xm, t = Y, rho = 0.9, lambda = 1, bias = "Multiplicative")
cholette(s = xq, t = Y, rho = 0.729, lambda = 1, bias = "Multiplicative")

# Proportional benchmarking with no bias
xm_no_bias <- ym_true + rnorm(n = length(ym_true), mean = 0, sd = 10000)
cholette(s = xm_no_bias, t = Y, rho = 0.9, lambda = 1)

# Additive benchmarking
cholette(s = xm, t = Y, rho = 0.9, lambda = 0, bias = "Additive")

# Denton PFD
cholette(s = xm, t = Y, rho = 1, lambda = 1)

# Pro-rating
cholette(s = xm, t = Y, rho = 0, lambda = 0.5)
\dontshow{\}) # examplesIf}
}
\references{
Quenneville, B., Fortier S., Chen Z.-G., Latendresse E. (2006).
Recent Developments in Benchmarking to Annual Totals in X12-ARIMA and at
Statistics Canada. Statistics Canada, Working paper of the Time Series
Research and Analysis Centre.
}
\seealso{
For more information, see the vignette:

\code{utils::browseVignettes()}, e.g. \code{browseVignettes(package = "rjd3bench")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/benchmark.R
\name{multivariatecholette}
\alias{multivariatecholette}
\title{Reconciliation by means of the Multivariate Cholette Method}
\usage{
multivariatecholette(
  xlist,
  tcvector = NULL,
  ccvector = NULL,
  rho = 0.8,
  lambda = 0.5
)
}
\arguments{
\item{xlist}{A named list of \code{ts} objects containing all input. Each element
should correspond to one input series: a preliminary series, a
low-frequency series representing a temporal aggregation constraint, or a
high-frequency series representing a contemporaneous constraint.}

\item{tcvector}{A character vector defining the temporal constraints. Each
element must be written in the form \code{"Y = sum(x)"}, where \code{"Y"} is the name
of a low-frequency temporal constraint and \code{"x"} is the name of a
high-frequency preliminary series. The names must match those provided in
\code{xlist}. The default is \code{NULL}, indicating that no temporal constraints are
considered.}

\item{ccvector}{NULL (default) or a character vector defining each contemporaneous constraints. If NULL, no contemporaneous constraint is considered.This is equivalent to applying the univariate Cholette method to each of the preliminary series separately. Otherwise, each element of the vector must be written in the form \eqn{z=w_1 x_1+\ldots+w_n x_n} or \eqn{c=w_1 x_1+\ldots+w_n x_n} where:
\itemize{
\item \eqn{z} is the name of a high-frequency contemporaneous constraint,
\item \eqn{(w_1,\ldots,w_n)} are optional numeric weights,
\item \eqn{(x_1,\ldots,x_n)} are the names of the high-frequency preliminary series and
\item \eqn{c} is a constant.
}

The \eqn{+} operator can be replaced by \eqn{-}. The names of the contemporaneous constraint(s) and the preliminary series are the one given in the \code{xlist} argument.

\strong{Important}: Any series placed on the left-hand side of a constraint cannot appear on the right-hand side of any other constraint. This is because quantities on the left-hand side are fixed, while those on the right-hand side are adjusted to satisfy the equality.}

\item{rho}{Numeric. The smoothing parameter whose value must lie between 0
and 1. The default is \code{0.8}. See the package vignette for more information
on the choice of the \code{rho} parameter.}

\item{lambda}{Numeric. The adjustment model parameter. Typical values include
\code{lambda = 0}, \code{lambda = 0.5} (the default) and \code{lambda = 1}. See the package
vignette for more information on the choice of the \code{lambda} parameter.}
}
\value{
A named list containing the benchmarked series is returned.
}
\description{
This is a multivariate extension of the Cholette benchmarking method which
can be used for the purpose of reconciliation. While standard benchmarking
methods consider one target series at a time, reconciliation techniques aim
to restore consistency in a system of time series with regards to both
contemporaneous and temporal constraints. Reconciliation techniques are
typically needed when the total and its components are estimated
independently (the so-called direct approach). The multivariate Cholette
method relies on the principle of movement preservation and encompasses other
reconciliation methods such as the multivariate Denton method.
}
\examples{
\dontshow{if (rjd3toolkit::get_java_version() >= rjd3toolkit::minimal_java_version) withAutoprint(\{ # examplesIf}
# Example 1: one "standard" contemporaneous constraint: x1+x2+x3 = z

x1 <- ts(c(7, 7.2, 8.1, 7.5, 8.5, 7.8, 8.1, 8.4), frequency = 4, start = c(2010, 1))
x2 <- ts(c(18, 19.5, 19.0, 19.7, 18.5, 19.0, 20.3, 20.0), frequency = 4, start = c(2010, 1))
x3 <- ts(c(1.5, 1.8, 2, 2.5, 2.0, 1.5, 1.7, 2.0), frequency = 4, start = c(2010, 1))

z <- ts(c(27.1, 29.8, 29.9, 31.2, 29.3, 27.9, 30.9, 31.8), frequency = 4, start = c(2010, 1))

Y1 <- ts(c(30.0, 30.6), frequency = 1, start = c(2010, 1))
Y2 <- ts(c(80.0, 81.2), frequency = 1, start = c(2010, 1))
Y3 <- ts(c(8.0, 8.1), frequency = 1, start = c(2010, 1))

## Check consistency between temporal and contemporaneous constraints
lfs <- cbind(Y1,Y2,Y3)
rowSums(lfs) - stats::aggregate.ts(z) # should all be 0

data_list <- list(x1 = x1, x2 = x2, x3 = x3, z = z, Y1 = Y1, Y2 = Y2, Y3 = Y3)
tc <- c("Y1 = sum(x1)", "Y2 = sum(x2)", "Y3 = sum(x3)") # temporal constraints
cc <- c("z = x1+x2+x3") # (binding) contemporaneous constraint
cc_nb <- c("0 = x1+x2+x3-z") # non-binding contemporaneous constraint

## Run function with default values for rho and lambda
multivariatecholette(xlist = data_list, tcvector = tc, ccvector = cc)

## Run function with some trade-off values for rho and lambda
multivariatecholette(xlist = data_list, tcvector = tc, ccvector = cc, rho = .5, lambda = .5)

## Run function with the value of rho corresponding to Denton or Cholette
multivariatecholette(xlist = data_list, tcvector = tc, ccvector = cc, rho = 1) # Denton
multivariatecholette(xlist = data_list, tcvector = tc, ccvector = cc, rho = 0.729) # Cholette

## Run function without temporal constraints
multivariatecholette(xlist = data_list, tcvector = NULL, ccvector = cc)

## Run function considering non-binding contemporaneous constraint
multivariatecholette(xlist = data_list, tcvector = tc, ccvector = cc_nb)

# Example 2: two contemporaneous constraints: x1+3*x2+0.5*x3+x4+x5 = z1 and x1+x2 = x4

x1 <- ts(c(7.0,7.3,8.1,7.5,8.5,7.8,8.1,8.4), frequency=4, start=c(2010,1))
x2 <- ts(c(1.5,1.8,2.0,2.5,2.0,1.5,1.7,2.0), frequency=4, start=c(2010,1))
x3 <- ts(c(18.0,19.5,19.0,19.7,18.5,19.0,20.3,20.0), frequency=4, start=c(2010,1))
x4 <- ts(c(8,9.5,9.0,10.7,8.5,10.0,10.3,9.0), frequency=4, start=c(2010,1))
x5 <- ts(c(5,9.6,7.2,7.1,4.3,4.6,5.3,5.9), frequency=4, start=c(2010,1))

z1 <- ts(c(38.1,41.8,41.9,43.2,38.8,39.1,41.9,43.7), frequency=4, start=c(2010,1))

Y1 <- ts(c(30.0,30.5), frequency=1, start=c(2010,1))
Y2 <- ts(c(10.0,10.5), frequency=1, start=c(2010,1))
Y3 <- ts(c(80.0,81.0), frequency=1, start=c(2010,1))
Y4 <- ts(c(40.0,41.0), frequency=1, start=c(2010,1))
Y5 <- ts(c(25.0,20.0), frequency=1, start=c(2010,1))

### check consistency between temporal and contemporaneous constraints
wlfs <- cbind(Y1,3*Y2,0.5*Y3,Y4,Y5)
rowSums(wlfs) - stats::aggregate.ts(z1) # cc1: should all be 0
Y1 + Y2 - Y4 # cc2: should all be 0

data.list <- list(x1=x1,x2=x2,x3=x3,x4=x4,x5=x5,z1=z1,Y1=Y1,Y2=Y2,Y3=Y3,Y4=Y4,Y5=Y5)
tc <- c("Y1=sum(x1)", "Y2=sum(x2)", "Y3=sum(x3)", "Y4=sum(x4)", "Y5=sum(x5)")
cc <- c("z1=x1+3*x2+0.5*x3+x4+x5", "0=x1+x2-x4")

multivariatecholette(xlist = data.list, tcvector = tc, ccvector = cc)
\dontshow{\}) # examplesIf}
}
\seealso{
For more information, see the vignette:

\code{utils::browseVignettes()}, e.g. \code{browseVignettes(package = "rjd3bench")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tempdisagg.R
\name{temporaldisaggregationI}
\alias{temporaldisaggregationI}
\title{Temporal Disaggregation and Interpolation of a Time Series by means of a Reverse Regression Model.}
\usage{
temporaldisaggregationI(
  series,
  indicator,
  conversion = c("Sum", "Average", "Last", "First", "UserDefined"),
  conversion.obsposition = 1L,
  rho = 0,
  rho.fixed = FALSE,
  rho.truncated = 0
)
}
\arguments{
\item{series}{A low-frequency time series to be disaggregated or interpolated. It must be a \code{"ts"} object.}

\item{indicator}{A high-frequency indicator series. It must be a \code{"ts"} object.}

\item{conversion}{A character string specifying the conversion mode, typically \code{"Sum"}(the default) or \code{"Average"}. Other options are: \code{"Last"}, \code{"First"} and \code{"UserDefined"}.}

\item{conversion.obsposition}{An integer specifying the position of the low-frequency observations within the interpolated series (e.g. the 7th month of the year).
This argument is used only for interpolation when \code{conversion = "UserDefined"}.}

\item{rho}{A numeric value giving the (initial) value of the autoregressive parameter.}

\item{rho.fixed}{Boolean. Specifies whether the supplied value of \code{rho} is fixed. The default is \code{FALSE}, which indicates that \code{rho} is estimated.}

\item{rho.truncated}{A numeric value defining the lower bound of the admissible range for \code{rho}.
The evaluation range is \verb{[rho.truncated, 1[}.}
}
\value{
An object of class "JD3_TEMPDISAGGI_RSLTS" is returned. The following are returned
invisibly as a list:
\itemize{
\item \code{regression} \verb{[[1]]} regression coefficients;
\item \code{estimation} \verb{[[2]]} disaggregated Time-Series and parameter;
\item \code{likelihood} \verb{[[3]]} likelihood statistics.
}
}
\description{
Perform temporal disaggregation and interpolation of low-frequency to high
frequency time series by means of a reverse regression model. Unlike the
usual regression-based models, this approach treats a high-frequency
indicator as the dependent variable and the unknown target series as the
independent variable.
}
\examples{
\dontshow{if (rjd3toolkit::get_java_version() >= rjd3toolkit::minimal_java_version) withAutoprint(\{ # examplesIf}
# Retail data, monthly indicator
Y <- rjd3toolkit::aggregate(rjd3toolkit::Retail$RetailSalesTotal, 1)
x <- rjd3toolkit::Retail$FoodAndBeverageStores
td <- temporaldisaggregationI(Y, indicator = x)
td$estimation$disagg

# qna data, quarterly indicator
data("qna_data")
Y <- ts(qna_data$B1G_Y_data[,"B1G_CE"], frequency = 1, start = c(2009,1))
x <- ts(qna_data$TURN_Q_data[,"TURN_INDEX_CE"], frequency = 4, start = c(2009,1))
td <- temporaldisaggregationI(Y, indicator = x)
td$regression$a
td$regression$b
\dontshow{\}) # examplesIf}
}
\references{
Bournay J., Laroque G. (1979). Reflexions sur la methode
d'elaboration des comptes trimestriels. Annales de l'Insee, n. 36, pp.3-30.
}
\seealso{
For more information, see the vignette:

\code{utils::browseVignettes()}, e.g. \code{browseVignettes(package = "rjd3bench")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/benchmark.R
\name{denton}
\alias{denton}
\title{Benchmarking by means of the Denton Method.}
\usage{
denton(
  s = NULL,
  t,
  d = 1L,
  mul = TRUE,
  nfreq = 4L,
  modified = TRUE,
  conversion = c("Sum", "Average", "Last", "First", "UserDefined"),
  obsposition = 1L,
  nbcsts = 0L,
  nfcsts = 0L
)
}
\arguments{
\item{s}{A preliminary series. If not \code{NULL}, it must be of the same class as \code{t}.}

\item{t}{The low-frequency aggregation constraint. It must be either a \code{"ts"} object or a numeric vector.}

\item{d}{An integer specifying the differencing order. The default is \code{1}.}

\item{mul}{Boolean. Indicates whether benchmarking is multiplicative (\code{TRUE}) or additive (\code{FALSE}). The default is multiplicative.}

\item{nfreq}{An integer giving the annual frequency of the benchmarked series.
This argument is used only when no preliminary series is provided.}

\item{modified}{Boolean. Specifies whether the modified Denton method (\code{TRUE}) or the unmodified Denton method (\code{FALSE}) is applied. The default is \code{TRUE}.}

\item{conversion}{A character string specifying the conversion mode, typically \code{"Sum"} (the default) or \code{"Average"}. Other options are: \code{"Last"}, \code{"First"} and \code{"UserDefined"}.}

\item{obsposition}{An integer specifying the position of the observations of the low-frequency constraint within the benchmarked series (e.g. the 7th month of the year).
This argument is used only when \code{conversion = "UserDefined"}.}

\item{nbcsts}{An integer specifying the number of backcast periods.
This argument is ignored when a preliminary series is provided.
(Not yet implemented.)}

\item{nfcsts}{An integer specifying the number of forecast periods.
This argument is ignored when a preliminary series is provided.
(Not yet implemented.)}
}
\value{
A \code{"ts"} object with the benchmarked series is returned.
}
\description{
Denton method relies on the principle of movement preservation. There exist a
few variants corresponding to different definitions of movement preservation:
additive first difference (AFD), proportional first difference (PFD),
additive second difference (ASD), proportional second difference (PSD), etc.
The default and most widely used is the Denton PFD method.
}
\examples{
\dontshow{if (rjd3toolkit::get_java_version() >= rjd3toolkit::minimal_java_version) withAutoprint(\{ # examplesIf}
Y <- rjd3toolkit::aggregate(rjd3toolkit::Retail$RetailSalesTotal, 1)

# Denton PFD without a preliminary series
y1 <- denton(t = Y, nfreq = 4)
print(y1)

# Denton PFD without a preliminary series and conversion = "Average"
denton(t = Y, nfreq = 4, conversion = "Average")

# Denton PFD with a preliminary series
x <- y1 + rnorm(n = length(y1), mean = 0, sd = 10000)
denton(s = x, t = Y)

# Denton AFD with a preliminary series
denton(s = x, t = Y, mul = FALSE)
\dontshow{\}) # examplesIf}
}
\seealso{
For more information, see the vignette:

\code{utils::browseVignettes()}, e.g. \code{browseVignettes(package = "rjd3bench")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\name{deprecated-rjd3bench}
\alias{deprecated-rjd3bench}
\alias{temporaldisaggregation}
\title{Deprecated Functions}
\usage{
temporaldisaggregation(
  series,
  constant = TRUE,
  trend = FALSE,
  indicators = NULL,
  model = c("Ar1", "Rw", "RwAr1"),
  freq = 4L,
  conversion = c("Sum", "Average", "Last", "First", "UserDefined"),
  conversion.obsposition = 1L,
  rho = 0,
  rho.fixed = FALSE,
  rho.truncated = 0,
  zeroinitialization = FALSE,
  diffuse.algorithm = c("SqrtDiffuse", "Diffuse", "Augmented"),
  diffuse.regressors = FALSE
)
}
\arguments{
\item{series, constant, trend, indicators, model, freq, conversion, conversion.obsposition, rho, rho.fixed, rho.truncated, zeroinitialization, diffuse.algorithm, diffuse.regressors}{Parameters.}
}
\value{
Return the same value as either function that replaces it.
}
\description{
This function is deprecated. Use the function \code{temporal_disaggregation()} or \code{temporal_interpolation()} instead.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tempdisagg.R
\name{temporal_disaggregation}
\alias{temporal_disaggregation}
\title{Temporal Disaggregation of a Time Series by Regression Models.}
\usage{
temporal_disaggregation(
  series,
  constant = TRUE,
  trend = FALSE,
  indicators = NULL,
  model = c("Ar1", "Rw", "RwAr1"),
  freq = 4L,
  average = FALSE,
  rho = 0,
  rho.fixed = FALSE,
  rho.truncated = 0,
  zeroinitialization = FALSE,
  diffuse.algorithm = c("SqrtDiffuse", "Diffuse", "Augmented"),
  diffuse.regressors = FALSE,
  nbcsts = 0L,
  nfcsts = 0L
)
}
\arguments{
\item{series}{A low-frequency time series to be disaggregated. It must be a \code{"ts"} object.}

\item{constant}{Boolean. Indicates whether a constant term is included in the model. The default is \code{TRUE}.
Note that this argument is used only with \code{model = "Ar1"} when \code{zeroinitialization = FALSE}. For additional information, see the package vignette.}

\item{trend}{Boolean. Indicates whether a linear trend is included in the model. The default is \code{FALSE}.}

\item{indicators}{One or more high-frequency indicator series used in the temporal disaggregation.
If \code{NULL} (the default), no indicator is used. When provided, the argument must be a \code{"ts"} object or a list of \code{"ts"} objects.}

\item{model}{A character string specifying the model of the error term at the disaggregated level.
The options are: \code{"Ar1"} (Chow Lin, the default), \code{"Rw"} (Fernandez), and \code{"RwAr1"} (Litterman).}

\item{freq}{An integer giving the annual frequency of the disaggregated series.
This argument is ignored when one or more indicator series is provided.}

\item{average}{Boolean. Indicates whether an average conversion should be considered. The default is \code{FALSE}, corresponding to additive conversion.}

\item{rho}{A numeric value giving the (initial) value of the autoregressive parameter.
This argument is used only for \code{"Ar1"} and \code{"RwAr1"} models.}

\item{rho.fixed}{Boolean. Specifies whether the supplied value of \code{rho} is fixed. The default is \code{FALSE}, which indicates that \code{rho} is estimated.}

\item{rho.truncated}{A numeric value defining the lower bound of the admissible range for \code{rho}.
The evaluation range is \verb{[rho.truncated, 1[}.}

\item{zeroinitialization}{Boolean. If \code{TRUE}, the initial values of the autoregressive model are set to zero. The default is \code{FALSE}.}

\item{diffuse.algorithm}{A character string specifying the algorithm used for diffuse initialization. The default is \code{"SqrtDiffuse"}. Other options are: \code{"Diffuse"} and \code{"Augmented"}.}

\item{diffuse.regressors}{Boolean. Indicates whether the coefficients of the regression model are treated as diffuse (\code{TRUE}) or as fixed unknown (\code{FALSE}, the default).}

\item{nbcsts}{An integer specifying the number of backcast periods.
This argument is ignored when one or more indicator series is provided.}

\item{nfcsts}{An integer specifying the number of forecast periods.
This argument is ignored when one or more indicator series is provided.}
}
\value{
An object of class "JD3_TEMPDISAGG_RSLTS" is returned. The following are returned
invisibly as a list:
\itemize{
\item \code{regression} \verb{[[1]]} regression coefficients;
\item \code{estimation} \verb{[[2]]} disaggregated Time-Series and standard deviation, regression effects, smoothing part, parameter and residuals;
\item \code{likelihood} \verb{[[3]]} likelihood statistics.
}
}
\description{
Perform temporal disaggregation of low-frequency to high-frequency time
series by regression models. The implemented models include Chow-Lin,
Fernandez, Litterman and some variants of those algorithms.
}
\examples{
\dontshow{if (rjd3toolkit::get_java_version() >= rjd3toolkit::minimal_java_version) withAutoprint(\{ # examplesIf}
# Chow-lin with a monthly indicator
Y <- rjd3toolkit::aggregate(rjd3toolkit::Retail$RetailSalesTotal, 1)
x <- rjd3toolkit::Retail$FoodAndBeverageStores
td <- temporal_disaggregation(Y, indicators = x)
td$estimation$disagg

# Fernandez with and without a quarterly indicator
data("qna_data")
Y <- ts(qna_data$B1G_Y_data[,"B1G_FF"], frequency = 1, start = c(2009,1))
x <- ts(qna_data$TURN_Q_data[,"TURN_INDEX_FF"], frequency = 4, start = c(2009,1))
td1 <- temporal_disaggregation(Y, indicators = x, model = "Rw")
td1$estimation$disagg

td2 <- temporal_disaggregation(Y, model = "Rw", nfcsts = 6)
td2$estimation$disagg

# Chow-lin applied to index series
Y_index <- 100 * Y / Y[1]
x_index <- 100 * x / x[1]
td3 <- temporal_disaggregation(Y, indicators = x, average = TRUE)
td3$estimation$disagg
\dontshow{\}) # examplesIf}
}
\seealso{
\code{temporal_interpolation()} for interpolation,

\code{temporal_disaggregation_raw()} for temporal disaggregation of atypical frequency series,

\code{temporal_interpolation_raw()} for interpolation of atypical frequency series

For more information, see the vignette:

\code{utils::browseVignettes()}, e.g. \code{browseVignettes(package = "rjd3bench")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/benchmark.R
\name{grp}
\alias{grp}
\title{Benchmarking following the Growth Rate Preservation Principle.}
\usage{
grp(
  s,
  t,
  objective = c("Forward", "Backward", "Symmetric", "Log"),
  conversion = c("Sum", "Average", "Last", "First", "UserDefined"),
  obsposition = 1L,
  eps = 1e-12,
  iter = 500L,
  dentoninitialization = TRUE
)
}
\arguments{
\item{s}{A preliminary series. It must be a \code{"ts"} object.}

\item{t}{The low-frequency aggregation constraint. It must be a \code{"ts"} object.}

\item{objective}{A character string specifying the objective function. The default is \code{"Forward"}. Other options are: \code{"Backward"}, \code{"Symmetric"} and \code{"Log"}. For additional information on this, see the package vignette.}

\item{conversion}{A character string specifying the conversion mode, typically \code{"Sum"} (the default) or \code{"Average"}. Other options are: \code{"Last"}, \code{"First"} and \code{"UserDefined"}.}

\item{obsposition}{An integer specifying the position of the observations of the low-frequency constraint within the benchmarked series (e.g. the 7th month of the year).
This argument is used only when \code{conversion = "UserDefined"}.}

\item{eps}{A numeric value specifying the convergence tolerance. The BFGS
algorithm proceeds until the reduction in the objective function is within
this tolerance (default is \code{1e-12}) or until the maximum number of iterations is reached.}

\item{iter}{An integer giving the maximum number of iterations allowed in the BFGS algorithm.
The default is \code{500}.}

\item{dentoninitialization}{Boolean. Indicates whether the series obtained
via the modified Denton PFD method is used as the starting values for the
GRP optimization procedure. The default is \code{TRUE}. If \code{FALSE}, the starting
values are derived directly from the aggregation constraint (e.g. \code{t/4} for
quarterly series with annual constraint and \code{conversion = "Sum"}).}
}
\value{
A \code{"ts"} object with the benchmarked series is returned.
}
\description{
GRP is a method which explicitly preserves the period-to-period growth rates
of the preliminary series. It corresponds to the method of Cauley and Trager
(1981), using the solution proposed by Di Fonzo and Marini (2011). BFGS is
used as line-search algorithm for the reduced unconstrained minimization
problem.
}
\examples{
\dontshow{if (rjd3toolkit::get_java_version() >= rjd3toolkit::minimal_java_version) withAutoprint(\{ # examplesIf}
data("qna_data")

Y <- ts(qna_data$B1G_Y_data[, "B1G_FF"], frequency = 1, start = c(2009, 1))
x <- denton(t = Y, nfreq = 4) + rnorm(n = length(Y) * 4, mean = 0, sd = 10)
grp(s = x, t = Y)
\dontshow{\}) # examplesIf}
}
\references{
Causey, B., and Trager, M.L. (1981). Derivation of Solution to
the Benchmarking Problem: Trend Revision. Unpublished research notes, U.S.
Census Bureau, Washington D.C. Available as an appendix in Bozik and Otto
(1988).

Di Fonzo, T., and Marini, M. (2011). A Newton's Method for Benchmarking
Time Series according to a Growth Rates Preservation Principle. \emph{IMF
WP/11/179}.

Daalmans, J., Di Fonzo, T., Mushkudiani, N. and Bikker, R. (2018). Growth
Rates Preservation (GRP) temporal benchmarking: Drawbacks and alternative
solutions. \emph{Statistics Canada}.
}
\seealso{
For more information, see the vignette:

\code{utils::browseVignettes()}, e.g. \code{browseVignettes(package = "rjd3bench")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tempdisagg.R
\name{temporal_interpolation_raw}
\alias{temporal_interpolation_raw}
\title{Interpolation of an Atypical Frequency Series by Regression Models.}
\usage{
temporal_interpolation_raw(
  series,
  constant = TRUE,
  trend = FALSE,
  indicators = NULL,
  startoffset = 0L,
  model = c("Ar1", "Rw", "RwAr1"),
  freqratio,
  obsposition = -1L,
  rho = 0,
  rho.fixed = FALSE,
  rho.truncated = 0,
  zeroinitialization = FALSE,
  diffuse.algorithm = c("SqrtDiffuse", "Diffuse", "Augmented"),
  diffuse.regressors = FALSE,
  nbcsts = 0L,
  nfcsts = 0L
)
}
\arguments{
\item{series}{A low-frequency time series to be interpolated. It must be a numeric vector.}

\item{constant}{Boolean. Indicates whether a constant term is included in the model. The default is \code{TRUE}.
Note that this argument is used only with \code{model = "Ar1"} when \code{zeroinitialization = FALSE}. For additional information, see the package vignette.}

\item{trend}{Boolean. Indicates whether a linear trend is included in the model. The default is \code{FALSE}.}

\item{indicators}{One or more high‑frequency indicator series used in the interpolation.
If \code{NULL} (the default), no indicator is used. When provided, the argument must be a numeric vector or a matrix.}

\item{startoffset}{The number of initial observations in the indicator series that precede the start of the low-frequency series.
The value must be either 0 or a positive integer (default is 0). This argument is ignored when no indicator is provided.}

\item{model}{A character string specifying the model of the error term at the disaggregated level.
The options are: \code{"Ar1"} (Chow Lin, the default), \code{"Rw"} (Fernandez), and \code{"RwAr1"} (Litterman).}

\item{freqratio}{An integer specifying the frequency ratio between the interpolated series and the low-frequency series.
This argument is mandatory and must be a positive integer.}

\item{obsposition}{An integer specifying the position of the low-frequency
observations within the interpolated series (e.g. the 1st month of the
year, the 2d month, etc.). The value must be a positive integer or \code{-1}
(the default).The default value is equivalent to setting the value of the
parameter equal to the frequency of the series, meaning that the last value
of the interpolated series is consistent with the low-frequency series.}

\item{rho}{A numeric value giving the (initial) value of the autoregressive parameter.
This argument is used only for \code{"Ar1"} and \code{"RwAr1"} models.}

\item{rho.fixed}{Boolean. Specifies whether the supplied value of \code{rho} is fixed. The default is \code{FALSE}, which indicates that \code{rho} is estimated.}

\item{rho.truncated}{A numeric value defining the lower bound of the admissible range for \code{rho}.
The evaluation range is \verb{[rho.truncated, 1[}.}

\item{zeroinitialization}{Boolean. If \code{TRUE}, the initial values of the autoregressive model are set to zero. The default is \code{FALSE}.}

\item{diffuse.algorithm}{A character string specifying the algorithm used for diffuse initialization. The default is \code{"SqrtDiffuse"}. Other options are: \code{"Diffuse"} and \code{"Augmented"}.}

\item{diffuse.regressors}{Boolean. Indicates whether the coefficients of the regression model are treated as diffuse (\code{TRUE}) or as fixed unknown (\code{FALSE}, the default).}

\item{nbcsts}{An integer specifying the number of backcast periods.
This argument is ignored when one or more indicator series is provided.}

\item{nfcsts}{An integer specifying the number of forecast periods.
This argument is ignored when one or more indicator series is provided.}
}
\value{
An object of class "JD3_INTERPRAW_RSLTS" is returned. The following are returned
invisibly as a list:
\itemize{
\item \code{regression} \verb{[[1]]} regression coefficients;
\item \code{estimation} \verb{[[2]]} interpolated values and standard deviation, regression effects, smoothing part, parameter and residuals;
\item \code{likelihood} \verb{[[3]]} likelihood statistics.
}
}
\description{
Perform temporal interpolation of low-frequency to high-frequency time series
by regression models. The implemented models include Chow-Lin, Fernandez,
Litterman and some variants of those algorithms. The
\code{temporal_interpolation_raw()} function extends \code{temporal_interpolation()} by
allowing temporal interpolation for any frequency ratio.
}
\examples{
\dontshow{if (rjd3toolkit::get_java_version() >= rjd3toolkit::minimal_java_version) withAutoprint(\{ # examplesIf}
# Use of Chow-lin method to interpolate a biennial series with an annual
# indicator (the low-frequency series is consistent with the last value of the
# interpolated series)
Y <- stats::aggregate(rjd3toolkit::Retail$RetailSalesTotal, 0.5)
x <- stats::aggregate(rjd3toolkit::Retail$FoodAndBeverageStores, 1)
ti <- temporal_interpolation_raw(as.numeric(Y), indicators = as.numeric(x), freqratio = 2)
ti$estimation$interp

# Use of Fernandez method to interpolate a series without indicator
# considering a frequency ratio of 5 (the low-frequency series is consistent
# with the last value of the interpolated series). For example, Y2 could be a
# quinquennial series to interpolate annually.
Y2 <- c(500,510,525,520)
ti2 <- temporal_interpolation_raw(Y2, model = "Rw", freqratio = 5, nbcsts = 1, nfcsts = 2)
ti2$estimation$interp

# Same with an indicator, considering an offset in the latter
Y2 <- c(500,510,525,520)
x2 <- c(485,
        490, 492.5, 497.5, 520, 495,
        500, 502.5, 505, 527.5, 515,
        522.5, 517.5, 522.5, 545, 520,
        535, 515, 540, 565, 550)
ti3 <- temporal_interpolation_raw(Y2, indicators = x2, startoffset = 1, model = "Rw", freqratio = 5)
ti3$estimation$interp

# Same considering that the first value of the interpolated series is the one
# consistent with the low-frequency series
ti4 <- temporal_interpolation_raw(Y2, indicators = x2, startoffset = 1,
                                  model = "Rw", freqratio = 5, obsposition = 1)
ti4$estimation$interp
\dontshow{\}) # examplesIf}
}
\seealso{
\code{temporal_disaggregation_raw()}

For more information, see the vignette:

\code{utils::browseVignettes()}, e.g. \code{browseVignettes(package = "rjd3bench")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tempdisagg.R
\name{temporal_disaggregation_raw}
\alias{temporal_disaggregation_raw}
\title{Temporal Disaggregation of an Atypical Frequency Series by Regression Models.}
\usage{
temporal_disaggregation_raw(
  series,
  constant = TRUE,
  trend = FALSE,
  indicators = NULL,
  startoffset = 0L,
  model = c("Ar1", "Rw", "RwAr1"),
  freqratio,
  average = FALSE,
  rho = 0,
  rho.fixed = FALSE,
  rho.truncated = 0,
  zeroinitialization = FALSE,
  diffuse.algorithm = c("SqrtDiffuse", "Diffuse", "Augmented"),
  diffuse.regressors = FALSE,
  nbcsts = 0L,
  nfcsts = 0L
)
}
\arguments{
\item{series}{A low-frequency time series to be disaggregated. It must be a numeric vector.}

\item{constant}{Boolean. Indicates whether a constant term is included in the model. The default is \code{TRUE}.
Note that this argument is used only with \code{model = "Ar1"} when \code{zeroinitialization = FALSE}. For additional information on this, see the package vignette.}

\item{trend}{Boolean. Indicates whether a linear trend is included in the model. The default is \code{FALSE}.}

\item{indicators}{One or more high-frequency indicator series used in the temporal disaggregation.
If \code{NULL} (the default), no indicator is used. When provided, the argument must be a numeric vector or a matrix.}

\item{startoffset}{The number of initial observations in the indicator series that precede the start of the low-frequency series.
The value must be either 0 or a positive integer (default is 0). This argument is ignored when no indicator is provided.}

\item{model}{A character string specifying the model of the error term at the disaggregated level.
The options are: \code{"Ar1"} (Chow Lin, the default), \code{"Rw"} (Fernandez), and \code{"RwAr1"} (Litterman).}

\item{freqratio}{An integer specifying the frequency ratio between the disaggregated series and the low-frequency series.
This argument is mandatory and must be a positive integer.}

\item{average}{Boolean. Indicates whether an average conversion should be considered. The default is \code{FALSE}, corresponding to additive conversion.}

\item{rho}{A numeric value giving the (initial) value of the autoregressive parameter.
This argument is used only for \code{"Ar1"} and \code{"RwAr1"} models.}

\item{rho.fixed}{Boolean. Specifies whether the supplied value of \code{rho} is fixed. The default is \code{FALSE}, which indicates that \code{rho} is estimated.}

\item{rho.truncated}{A numeric value defining the lower bound of the admissible range for \code{rho}.
The evaluation range is \verb{[rho.truncated, 1[}.}

\item{zeroinitialization}{Boolean. If \code{TRUE}, the initial values of the autoregressive model are set to zero. The default is \code{FALSE}.}

\item{diffuse.algorithm}{A character string specifying the algorithm used for diffuse initialization. The default is \code{"SqrtDiffuse"}. Other options are: \code{"Diffuse"} and \code{"Augmented"}.}

\item{diffuse.regressors}{Boolean. Indicates whether the coefficients of the regression model are treated as diffuse (\code{TRUE}) or as fixed unknown (\code{FALSE}, the default).}

\item{nbcsts}{An integer specifying the number of backcast periods.
This argument is ignored when one or more indicator series is provided.}

\item{nfcsts}{An integer specifying the number of forecast periods.
This argument is ignored when one or more indicator series is provided.}
}
\value{
An object of class "JD3_TEMPDISAGGRAW_RSLTS" is returned. The following are returned
invisibly as a list:
\itemize{
\item \code{regression} \verb{[[1]]} regression coefficients;
\item \code{estimation} \verb{[[2]]} disaggregated values and standard deviation, regression effects, smoothing part, parameter and residuals;
\item \code{likelihood} \verb{[[3]]} likelihood statistics.
}
}
\description{
Perform temporal disaggregation of low-frequency to high-frequency time
series by regression models. The implemented models include Chow-Lin,
Fernandez, Litterman and some variants of those algorithms. The
\code{temporal_disaggregation_raw()} function extends \code{temporal_disaggregation()}
by allowing temporal disaggregation for any frequency ratio.
}
\examples{
\dontshow{if (rjd3toolkit::get_java_version() >= rjd3toolkit::minimal_java_version) withAutoprint(\{ # examplesIf}
# Use of Chow-lin method to disaggregate a biennial series with an annual indicator
Y <- stats::aggregate(rjd3toolkit::Retail$RetailSalesTotal, 0.5)
x <- stats::aggregate(rjd3toolkit::Retail$FoodAndBeverageStores, 1)
td <- temporal_disaggregation_raw(as.numeric(Y), indicators = as.numeric(x), freqratio = 2)
td$estimation$disagg

# Use of Fernandez method to disaggregate a series without indicator
# considering a frequency ratio of 5 (for example, it could be a quinquennial
# series to disaggregate on an annual basis)
Y2 <- c(500,510,525,520)
td2 <- temporal_disaggregation_raw(Y2, model = "Rw", freqratio = 5, nfcsts = 2)
td2$estimation$disagg

# Same with an indicator, considering an offset in the latter
Y2 <- c(500,510,525,520)
x2 <- c(97,
        98, 98.5, 99.5, 104, 99,
        100, 100.5, 101, 105.5, 103,
        104.5, 103.5, 104.5, 109, 104,
        107, 103, 108, 113, 110)
td3 <- temporal_disaggregation_raw(Y2, indicators = x2, startoffset = 1,
                                   model = "Rw", freqratio = 5)
td3$estimation$disagg
\dontshow{\}) # examplesIf}
}
\seealso{
\code{temporal_interpolation_raw()}

For more information, see the vignette:

\code{utils::browseVignettes()}, e.g. \code{browseVignettes(package = "rjd3bench")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calendarization.R
\name{calendarization}
\alias{calendarization}
\title{Calendarization}
\usage{
calendarization(
  calendarobs,
  freq,
  start = NULL,
  end = NULL,
  dailyweights = NULL,
  stde = FALSE
)
}
\arguments{
\item{calendarobs}{A named list containing the observed data. The list must
consist of three elements: \code{start}, \code{end} and \code{value}, where the first two
indicate the starting and ending dates of the observation. See the example.}

\item{freq}{An integer specifying the annual frequency. If set to \code{0}, only the daily series is computed.}

\item{start}{The starting day of the calendarization. This date may precede the first observed data (retropolation).}

\item{end}{The ending day of the calendarization. This date may exceed the last observed data (extrapolation).}

\item{dailyweights}{A numeric vector of daily indicator values (or weights).
The vector must have the same length as the requested daily series. When
available, these weights typically reflects daily levels of activity, which
may vary due to seasonality, trading day effects, or other calendar effects
such as public holidays.}

\item{stde}{Boolean. If \code{TRUE}, the function also returns the standard errors associated with the results.
The default is \code{FALSE}.}
}
\value{
A list containing the disaggregated daily values, the final aggregated series, and their associated standard errors if requested.
}
\description{
Time series data do not always coincide with calendar periods (e.g., fiscal
years starting in March-April or retail data collected in non-monthly
intervals). Calendarization is the process of transforming the values of a
flow time series observed over varying time intervals into values that cover
given calendar intervals such as month, quarter or year. The process involves
two steps. At first, a state-space representation of the Denton proportional
first difference (PFD) method is considered to perform a temporal
disaggregation of the observed data into daily values. After that, the
resulting daily values are aggregated into the desired calendar reference
periods.
}
\examples{
\dontshow{if (rjd3toolkit::get_java_version() >= rjd3toolkit::minimal_java_version) withAutoprint(\{ # examplesIf}

# Example 1 (from Quenneville et al (2012))

## Observed data
obs_1 <- list(
    list(start = "2009-02-18", end = "2009-03-17", value = 9000),
    list(start = "2009-03-18", end = "2009-04-14", value = 5000),
    list(start = "2009-04-15", end = "2009-05-12", value = 9500),
    list(start = "2009-05-13", end = "2009-06-09", value = 7000))

## a) calendarization in absence of daily indicator values (or weights)
cal_1a <- calendarization(obs_1, 12, end = "2009-06-30", dailyweights = NULL, stde = TRUE)

ym_1a <- cal_1a$rslt
eym_1a <- cal_1a$erslt
yd_1a <- cal_1a$days
eyd_1a <- cal_1a$edays

## b) calendarization in presence of daily indicator values (or weights)
x <- rep(c(1.0, 1.2, 1.8 , 1.6, 0.0, 0.6, 0.8), 19)
cal_1b <- calendarization(obs_1, 12, end = "2009-06-30", dailyweights = x, stde = TRUE)

ym_1b <- cal_1b$rslt
eym_1b <- cal_1b$erslt
yd_1b <- cal_1b$days
eyd_1b <- cal_1b$edays

# Example 2 (incl. negative value)

obs_2 <- list(
    list(start = "1980-01-01", end = "1989-12-31", value = 100),
    list(start = "1990-01-01", end = "1999-12-31", value = -10),
    list(start = "2000-01-01", end = "2002-12-31", value = 50))

cal_2 <- calendarization(obs_2, 4, end = "2003-12-31")

yq_2 <- cal_2$rslt
\dontshow{\}) # examplesIf}
}
\references{
Quenneville, B., Picard F., Fortier S. (2012). Calendarization
with interpolating splines and state space models. Statistics Canada, Appl.
Statistics (2013) 62, part 3, pp 371-399.
}
\seealso{
For more information, see the vignette:

\code{utils::browseVignettes()}, e.g. \code{browseVignettes(package = "rjd3bench")}
}
